
from abc import ABC, abstractmethod


class ConvProgressUpdator(ABC):
    def __init__(self):
        pass

    def update(self, progress: int) -> bool:
        # TODO ratelimie
        return self.do_update(progress)
    
    @abstractmethod
    def do_update(self, progress: int):
        pass
    




from io import BytesIO

from loguru import logger
from magic_pdf.dict2md.ocr_mkcontent import union_make
from magic_pdf.libs.MakeContentConfig import MakeMode, DropMode
from magic_pdf.libs.json_compressor import JsonCompressor
from werkzeug.datastructures import FileStorage

from magic_doc.contrib.pdf.pdf_extractor import PDFExtractor
from magic_doc.conv.base import BaseConv
from magic_doc.conv.pdf_magicpdf import SingletonModelWrapper
from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_doc.progress.pupdator import ConvProgressUpdator
from magic_doc.utils.null_writer import NullWriter
from magic_pdf.pipe.OCRPipe import OCRPipe

class Pdf(BaseConv):
    def to_md(self, bits: bytes | str, pupdator: ConvProgressUpdator) -> str:
        # TODO: 单例化模型
        pdf_extractor = PDFExtractor()
        buf = BytesIO(bits)  # type: ignore
        content = pdf_extractor.run("stream io data", FileStorage(buf, "fake.pdf"))
        arr = []
        pupdator.update(0)

        N = len(content)
        progress_h = {N * i // 100: 1 for i in range(10, 100, 10)}
        for idx, page in enumerate(content):
            if idx in progress_h:
                pupdator.update(idx * 100 // N)
            for record in page.get("content_list", []):
                arr.append(record.get("data", ""))

        text_all = ""
        for content in arr:
            text_all += content

        def calculate_not_printable_rate(text):
            printable = sum(1 for c in text if c.isprintable())
            total = len(text)
            if total == 0:
                return 0  # 避免除以零的错误
            return (total - printable) / total
        not_printable_rate = calculate_not_printable_rate(text_all)
        if not_printable_rate > 0.02:
            # pdf可能是乱码，切换到ocr处理
            logger.info("switch to ocrpipe by garbled check")
            model_proc = SingletonModelWrapper()
            model_list = model_proc(bits)  # type: ignore
            pupdator.update(50)
            image_writer = NullWriter()
            pipe = OCRPipe(bits, model_list, image_writer, is_debug=True)  # type: ignore
            pipe.pipe_parse()
            pdf_mid_data = JsonCompressor.decompress_json(pipe.get_compress_pdf_mid_data())
            pdf_info_list = pdf_mid_data["pdf_info"]
            NULL_IMG_DIR = "/tmp"
            md_content = union_make(pdf_info_list, MakeMode.NLP_MD, DropMode.NONE, NULL_IMG_DIR)
            pupdator.update(100)
            return md_content
        else:
            pupdator.update(100)
            return "\n\n".join(arr)


if __name__ == "__main__":
    if 1:
        with open("/opt/data/pdf/20240423/pdf_test2/ol006018w.pdf", "rb") as f:
            bits_data = f.read()
            parser = Pdf()
            md_content = parser.to_md(bits_data, FileBaseProgressUpdator("debug/progress.txt"))

        with open("debug/pdf2md.md", "w") as f:
            f.write(md_content)



import shutil
import json

from loguru import logger
from pathlib import Path
from subprocess import Popen, PIPE

from werkzeug.datastructures import FileStorage

from magic_doc.contrib.office import OfficeExtractor
from magic_doc.contrib.model import Page, Content, ExtractResponse


class DocExtractor(OfficeExtractor):
    def __init__(self) -> None:
        super().__init__()

    def setup(self):
        pass

    def extract(
        self,
        r: FileStorage | Path,
        id: str,
        dir: Path,
        media_dir: Path,
        skip_image: bool,
        cwd_path="/opt/antiword"
    ) -> ExtractResponse:
        doc_path = dir.joinpath("its.doc")

        if type(r) is FileStorage:
            r.save(doc_path)
        else:
            shutil.copyfile(r, doc_path)

        if skip_image:
            cmd = f"./antiword -f -i 1 -o {dir.as_posix()} {doc_path.as_posix()}"
        else:
            cmd = f"./antiword -f -i 3 -o {dir.as_posix()} {doc_path.as_posix()}"
        logger.info(f"cmd: {cmd}")
        process = Popen(cmd, shell=True, cwd=Path(cwd_path), stdout=PIPE, stderr=PIPE)
        stdout, stderr = process.communicate()
        process.wait()

        shutil.rmtree(media_dir.absolute().as_posix())
        shutil.move(
            dir.joinpath("pic").absolute().as_posix(), media_dir.absolute().as_posix()
        )
        code = process.returncode
        if code != 0:
            err = stderr.decode()
            raise Exception(f"parse doc failed: {err}")

        pure_text_path = dir.joinpath("text")

        with open(pure_text_path, "r") as f:
            content = f.read()

        # img_map: dict[Path, str] = {}
        # imgs = media_dir.glob("*")
        # for img in imgs:
        #     img_map[img] = self.generate_img_path(id, img.name)
        #
        # self.upload_background(id, img_map)

        pages = [
            Page(page_no=idx, content=x)
            for idx, x in enumerate(content.split("[pedia-page]"))
        ]

        for page in pages:
            content: str = page.pop("content")
            content_list = [
                Content(data=x.strip(), type="text")
                for x in content.split("\n")
                if x.strip()
            ]

            for content in content_list:
                if not content["data"].startswith("[pedia-"):
                    continue
                if content["data"] == "[pedia-badpic]":
                    content["data"] = ""
                    content["type"] = "image"
                elif content["data"].startswith("[pedia-pic"):
                    content["type"] = "image"
                    img_name = content["data"][len("[pedia-") : -1]
                    img_path = media_dir.joinpath(img_name)
                    content["data"] = img_map[img_path]
                else:
                    content["data"] = content["data"] + "\n"

            page["content_list"] = content_list

        return pages


if __name__ == "__main__":
    e = DocExtractor()
    print(
        json.dumps(
            e.run("abc", Path("/home/SENSETIME/wuziming/diclm/doc2docx/doc/md4.doc")),
            ensure_ascii=False,
            indent=4,
        ),
    )
    e.wait_all()




import os
import yaml
from collections import deque

def patch_yaml_load_with_env(yaml_file, env_namespace, loader=yaml.FullLoader):
    with open(yaml_file, "r") as f:
        configs = yaml.load(f, Loader=yaml.FullLoader)

    # patch config with envs
    for env_var in os.environ:
        arr = deque(map(lambda x: x.lower(), env_var.split("_")))
        if arr[0] != env_namespace:
            continue
        arr.popleft()
        d = configs
        while arr:
            if arr[0] not in d:
                break
            if len(arr) > 1:
                d = d[arr[0]]
                arr.popleft()
            else:
                d[arr[0]] = os.environ[env_var]
                break
    return configs



import json
import re
import time
import requests
from flask import request, current_app
from flask_restful import Resource
from marshmallow import ValidationError
from pathlib import Path
from magic_doc.pdf_transform import DocConverter, S3Config
from .serialization import MagicPdfSchema
from magic_pdf.dict2md.ocr_mkcontent import ocr_mk_mm_markdown_with_para_and_pagination
from magic_doc.restful_api.common.oss.oss import Oss
from .ext import upload_image_to_oss, upload_md_to_oss
from concurrent.futures import ThreadPoolExecutor, wait, ALL_COMPLETED
from magic_doc.restful_api.common.custom_response import generate_response
from loguru import logger

executor = ThreadPoolExecutor()


class MagicPdfView(Resource):
    @logger.catch
    def post(self):
        """
        PDF解析，将markdown结果上传至服务器
        """
        t0 = time.time()
        magic_pdf_schema = MagicPdfSchema()
        try:
            params = magic_pdf_schema.load(request.get_json())
        except ValidationError as err:
            return generate_response(code=400, msg=err.messages)
        pdf_path = params.get('pageUrl')
        # ############ pdf解析  ###############
        file_name = str(Path(pdf_path).stem)
        pf_path = f"/tmp/{file_name}.txt"
        pdf_dir = f"{current_app.static_folder}/pdf/{file_name}"
        NULL_IMG_DIR = f"{current_app.static_folder}/pdf/{file_name}"
        app_config = current_app.config
        if not Path(NULL_IMG_DIR).exists():
            Path(NULL_IMG_DIR).mkdir(parents=True, exist_ok=True)
        if pdf_path.startswith("http://") or pdf_path.startswith("https://"):
            download_pdf = requests.get(pdf_path, stream=True)
            pdf_path = f"{pdf_dir}/{file_name}.pdf"
            with open(pdf_path, "wb") as wf:
                wf.write(download_pdf.content)
            doc_conv = DocConverter(None)
        elif pdf_path.startswith("s3://"):
            s3_config = S3Config(app_config["S3AK"], app_config["S3SK"], app_config["S3ENDPOINT"])
            doc_conv = DocConverter(s3_config)
        else:
            doc_conv = DocConverter(None)
        t1 = time.time()
        logger.info(f"param init cost_time:{t1 - t0}")
        result = doc_conv.convert_to_mid_result(pdf_path, pf_path, 60)
        t2 = time.time()
        logger.info(f"pdf doc_conv cost_time:{t2 - t1}")
        md_content = json.dumps(ocr_mk_mm_markdown_with_para_and_pagination(result[0], NULL_IMG_DIR), ensure_ascii=False)
        t3 = time.time()
        logger.info(f"make markdown cost_time:{t3 - t2}")
        # local_md_path = f"{pdf_dir}/{file_name}.md"
        # with open(local_md_path, "w", encoding="utf-8") as f:
        #     f.write(md_content)
        # t4 = time.time()
        # logger.info(f"save markdown cost_time:{t4 - t3}")
        _t0 = time.time()
        oss_client = Oss(
            app_config["AccessKeyID"],
            app_config["AccessKeySecret"],
            app_config["BucketName"],
            app_config["Endpoint"],
            app_config["UrlExpires"]
        )
        img_list = Path(f"{NULL_IMG_DIR}/images").glob('*') if Path(f"{NULL_IMG_DIR}/images").exists() else []
        all_task = [executor.submit(upload_image_to_oss, oss_client, file_name, img_path, NULL_IMG_DIR, app_config["BucketName"]) for img_path in img_list]
        wait(all_task, return_when=ALL_COMPLETED)
        for task in all_task:
            task_result = task.result()
            regex = re.compile(fr'.*\((.*?{Path(task_result[0]).name})')
            regex_result = regex.search(md_content)
            if regex_result:
                md_content = md_content.replace(regex_result.group(1), task_result[1])
        _t1 = time.time()
        logger.info(f"upload img cost_time:{_t1 - _t0}")

        all_md_task = [executor.submit(upload_md_to_oss, oss_client, app_config["BucketName"], f"pdf/{file_name}/{md.get('page_no', n)}.md", md["md_content"]) for n, md in enumerate(json.loads(md_content))]
        wait(all_md_task, return_when=ALL_COMPLETED)
        md_link_list = []
        for task in all_md_task:
            task_result = task.result()
            md_link_list.append(task_result)
        _t2 = time.time()
        logger.info(f"upload md cost_time:{_t2 - _t1}")

        return generate_response(markDownUrl=md_link_list)



import lxml
from marshmallow import Schema, fields, validates, ValidationError


class MagicHtmlSchema(Schema):
    pageUrl = fields.Str()
    html = fields.Str(required=True)
    html_type = fields.Str()

    @validates('html')
    def validate_html(self, data, **kwargs):
        if not data:
            raise ValidationError('HTML cannot be empty')
        else:
            if lxml.html.fromstring(data).find('.//*') is None:
                raise ValidationError('Content is not HTML')
            return data


class MagicPdfSchema(Schema):
    pageUrl = fields.Str(required=True)

    @validates('pageUrl')
    def validate_url(self, data, **kwargs):
        if not data:
            raise ValidationError('pageUrl cannot be empty')
        else:
            return data



import os

from lxml import etree as ET

transform = None

_xslt_filename = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "xsl/mmltex.xsl"
)


def mml2tex(mml_xml):
    tree = ET.fromstring(mml_xml)
    global transform
    if not transform:
        transform = ET.XSLT(ET.parse(_xslt_filename))
    return str(transform(tree))



from abc import ABC
from pathlib import Path
from werkzeug.datastructures import FileStorage

from magic_doc.contrib.model import ExtractResponse, Extractor


class HtmlExtractor(Extractor, ABC):
    def __init__(self) -> None:
        super().__init__()

    def setup(self):
        pass

    def run(id: str, r: FileStorage | Path, skip_image: bool = True) -> ExtractResponse:
        return []



import requests
import json

from pathlib import Path


from werkzeug.datastructures import FileStorage

from magic_doc.contrib.office import OfficeExtractor
from magic_doc.contrib.model import ExtractResponse


class PptExtractor(OfficeExtractor):
    def __init__(self) -> None:
        super().__init__()

    def setup(self):
        pass

    def extract(
        self,
        r: FileStorage | Path,
        id: str,
        dir: Path,
        media_dir: Path,
        skip_image: bool,
    ) -> ExtractResponse:

        if type(r) is FileStorage:
            data = r.stream.read()
        elif issubclass(type(r), Path):
            with open(r, "rb") as data_file:
                data = data_file.read()

        files = {"file": data}
        response = requests.post(f"{self.config.tika}/api/v1/parse", files=files)
        self.upload_background(id, {})
        return response.json()["pages"]


if __name__ == "__main__":
    e = PptExtractor()
    print(
        json.dumps(
            e.run(
                "def",
                Path(
                    "/home/SENSETIME/wuziming/diclm/doc2docx/doc/【中繁-课件】物理学简介.ppt",
                ),
            ),
            ensure_ascii=False,
            indent=4,
        )
    )
    e.wait_all()



import json
import sys

from pathlib import Path
from typing import List

from loguru import logger
from pptx import Presentation
from pptx.enum.shapes import MSO_SHAPE_TYPE
from pptx.parts.image import Image
from pptx.presentation import Presentation as ppt
from pptx.shapes.autoshape import Shape
from pptx.shapes.picture import Picture
from pptx.shapes.graphfrm import GraphicFrame
from pptx.table import Table, _Row, _Cell
from pptx.slide import Slide
from pptx.shapes.group import GroupShape
from werkzeug.datastructures import FileStorage

from magic_doc.contrib.office import OfficeExtractor
from magic_doc.contrib.model import ExtractResponse, Page, Content


class PptxExtractor(OfficeExtractor):
    def __init__(self) -> None:
        super().__init__()

    def setup(self):
        pass

    def handle_shape(
        self,
        shape: Shape,
        content_list: List[Content],
        media_dir: Path,
        img_map: dict[Path, str],
        id: str,
        skip_image: bool,
    ):
        if shape.has_text_frame:
            for paragraph in shape.text_frame.paragraphs:
                content_list.append(
                    Content(
                        type="text",
                        data=paragraph.text + "\n",
                    )
                )
        elif shape.shape_type == MSO_SHAPE_TYPE.PICTURE and not skip_image:
            shape: Picture
            image: Image = shape.image
            image_bytes = image.blob
            img_path = media_dir.joinpath(f"pic-{len(img_map)}.{image.ext}")
            img_s3_path = self.generate_img_path(id, img_path.name)
            img_map[img_path] = img_s3_path
            content_list.append(Content(type="image", data=img_s3_path))
            with open(img_path, "wb") as file:
                file.write(image_bytes)
        elif shape.shape_type == MSO_SHAPE_TYPE.TABLE:
            shape: GraphicFrame
            table: Table = shape.table
            md = "\n"
            for row_no, row in enumerate(table.rows):
                row: _Row
                md += "|"
                if row_no == 1:
                    for col in row.cells:
                        md += "---|"
                    md += "\n|"
                for col in row.cells:
                    cell: _Cell = col
                    md += " " + cell.text.replace("\r", " ").replace("\n", " ") + " |"
                md += "\n"
            md += "\n"
            content_list.append(Content(type="md", data=md))
        elif shape.shape_type == MSO_SHAPE_TYPE.GROUP:
            shape: GroupShape
            for sub_shape in shape.shapes:
                self.handle_shape(sub_shape, content_list, media_dir, img_map, id, skip_image)
        else:
            # print(shape.shape_type, type(shape), file=sys.stderr)
            pass

    def extract(
        self,
        r: FileStorage | Path,
        id: str,
        dir: Path,
        media_dir: Path,
        skip_image: bool,
    ) -> ExtractResponse:
        pages = []
        img_map = {}

        presentation: ppt = Presentation(r)
        for page_no, slide in enumerate(presentation.slides):
            slide: Slide
            page = Page(page_no=page_no, content_list=[])
            for shape in slide.shapes:
                self.handle_shape(
                    shape,
                    page["content_list"],
                    media_dir,
                    img_map,
                    id,
                    skip_image,
                )

            pages.append(page)

        # self.upload_background(id, img_map)

        return pages


if __name__ == "__main__":
    e = PptxExtractor()
    # from pedia_document_parser.s3.client import get_s3_client

    # cli = get_s3_client()

    # data = cli.read_object(
    #     "s3://pedia-document-parser/office-doucments/【英文-模板】Professional Pack Standard.pptx"
    # )
    # with open("1.pptx", "wb") as f:
    #     f.write(data.read())

    x = e.run(
        "ghi",
        Path("test_data/doc/商业项目市场分析与产品定位报告.pptx"),
    )
    content = ""
    for p in x:
        content += f"\n====== page {p['page_no']} ======\n"
        for pp in p["content_list"]:
            content += pp["data"]

    print(content)

    # cli.read_object("s3://pedia-document-parser/office-doucments/【英文-课件】MIT15_082JF10_av.pptx")

    # print(
    #     json.dumps(
    #         e.run(
    #             "ghi",
    #             Path(
    #                 "/home/SENSETIME/wuziming/doc/doc/【英文-模板】Professional Pack Standard.pptx",
    #             ),
    #         ),
    #         ensure_ascii=False,
    #         indent=4,
    #     )
    # )
    e.wait_all()




from abc import ABC, abstractmethod

from magic_doc.progress.pupdator import ConvProgressUpdator
from magic_pdf.rw.AbsReaderWriter import AbsReaderWriter
class BaseConv(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def to_md(self, bits: bytes | str, pupdator:ConvProgressUpdator) -> str:
        return NotImplemented

    def to_mid_result(self, rw: AbsReaderWriter,  bits: bytes | str, pupdator:ConvProgressUpdator) -> list[dict] | dict:
        pupdator.update(100)
        return {}



# --------------------------------------------------------------------------------
# VIT: Multi-Path Vision Transformer for Dense Prediction
# Copyright (c) 2022 Electronics and Telecommunications Research Institute (ETRI).
# All Rights Reserved.
# Written by Youngwan Lee
# This source code is licensed(Dual License(GPL3.0 & Commercial)) under the license found in the
# LICENSE file in the root directory of this source tree.
# --------------------------------------------------------------------------------
# References:
# timm: https://github.com/rwightman/pytorch-image-models/tree/master/timm
# CoaT: https://github.com/mlpc-ucsd/CoaT
# --------------------------------------------------------------------------------


import torch

from detectron2.layers import (
    ShapeSpec,
)
from detectron2.modeling import Backbone, BACKBONE_REGISTRY, FPN
from detectron2.modeling.backbone.fpn import LastLevelP6P7, LastLevelMaxPool

from .beit import beit_base_patch16, dit_base_patch16, dit_large_patch16, beit_large_patch16
from .deit import deit_base_patch16, mae_base_patch16
from .layoutlmft.models.layoutlmv3 import LayoutLMv3Model
from transformers import AutoConfig

__all__ = [
    "build_vit_fpn_backbone",
]


class VIT_Backbone(Backbone):
    """
    Implement VIT backbone.
    """

    def __init__(self, name, out_features, drop_path, img_size, pos_type, model_kwargs,
                 config_path=None, image_only=False, cfg=None):
        super().__init__()
        self._out_features = out_features
        if 'base' in name:
            self._out_feature_strides = {"layer3": 4, "layer5": 8, "layer7": 16, "layer11": 32}
            self._out_feature_channels = {"layer3": 768, "layer5": 768, "layer7": 768, "layer11": 768}
        else:
            self._out_feature_strides = {"layer7": 4, "layer11": 8, "layer15": 16, "layer23": 32}
            self._out_feature_channels = {"layer7": 1024, "layer11": 1024, "layer15": 1024, "layer23": 1024}

        if name == 'beit_base_patch16':
            model_func = beit_base_patch16
        elif name == 'dit_base_patch16':
            model_func = dit_base_patch16
        elif name == "deit_base_patch16":
            model_func = deit_base_patch16
        elif name == "mae_base_patch16":
            model_func = mae_base_patch16
        elif name == "dit_large_patch16":
            model_func = dit_large_patch16
        elif name == "beit_large_patch16":
            model_func = beit_large_patch16

        if 'beit' in name or 'dit' in name:
            if pos_type == "abs":
                self.backbone = model_func(img_size=img_size,
                                           out_features=out_features,
                                           drop_path_rate=drop_path,
                                           use_abs_pos_emb=True,
                                           **model_kwargs)
            elif pos_type == "shared_rel":
                self.backbone = model_func(img_size=img_size,
                                           out_features=out_features,
                                           drop_path_rate=drop_path,
                                           use_shared_rel_pos_bias=True,
                                           **model_kwargs)
            elif pos_type == "rel":
                self.backbone = model_func(img_size=img_size,
                                           out_features=out_features,
                                           drop_path_rate=drop_path,
                                           use_rel_pos_bias=True,
                                           **model_kwargs)
            else:
                raise ValueError()
        elif "layoutlmv3" in name:
            config = AutoConfig.from_pretrained(config_path)
            # disable relative bias as DiT
            config.has_spatial_attention_bias = False
            config.has_relative_attention_bias = False
            self.backbone = LayoutLMv3Model(config, detection=True,
                                               out_features=out_features, image_only=image_only)
        else:
            self.backbone = model_func(img_size=img_size,
                                       out_features=out_features,
                                       drop_path_rate=drop_path,
                                       **model_kwargs)
        self.name = name

    def forward(self, x):
        """
        Args:
            x: Tensor of shape (N,C,H,W). H, W must be a multiple of ``self.size_divisibility``.

        Returns:
            dict[str->Tensor]: names and the corresponding features
        """
        if "layoutlmv3" in self.name:
            return self.backbone.forward(
                input_ids=x["input_ids"] if "input_ids" in x else None,
                bbox=x["bbox"] if "bbox" in x else None,
                images=x["images"] if "images" in x else None,
                attention_mask=x["attention_mask"] if "attention_mask" in x else None,
                # output_hidden_states=True,
            )
        assert x.dim() == 4, f"VIT takes an input of shape (N, C, H, W). Got {x.shape} instead!"
        return self.backbone.forward_features(x)

    def output_shape(self):
        return {
            name: ShapeSpec(
                channels=self._out_feature_channels[name], stride=self._out_feature_strides[name]
            )
            for name in self._out_features
        }


def build_VIT_backbone(cfg):
    """
    Create a VIT instance from config.

    Args:
        cfg: a detectron2 CfgNode

    Returns:
        A VIT backbone instance.
    """
    # fmt: off
    name = cfg.MODEL.VIT.NAME
    out_features = cfg.MODEL.VIT.OUT_FEATURES
    drop_path = cfg.MODEL.VIT.DROP_PATH
    img_size = cfg.MODEL.VIT.IMG_SIZE
    pos_type = cfg.MODEL.VIT.POS_TYPE

    model_kwargs = eval(str(cfg.MODEL.VIT.MODEL_KWARGS).replace("`", ""))

    if 'layoutlmv3' in name:
        if cfg.MODEL.CONFIG_PATH != '':
            config_path = cfg.MODEL.CONFIG_PATH
        else:
            config_path = cfg.MODEL.WEIGHTS.replace('pytorch_model.bin', '')  # layoutlmv3 pre-trained models
            config_path = config_path.replace('model_final.pth', '')  # detection fine-tuned models
    else:
        config_path = None

    return VIT_Backbone(name, out_features, drop_path, img_size, pos_type, model_kwargs,
                        config_path=config_path, image_only=cfg.MODEL.IMAGE_ONLY, cfg=cfg)


@BACKBONE_REGISTRY.register()
def build_vit_fpn_backbone(cfg, input_shape: ShapeSpec):
    """
    Create a VIT w/ FPN backbone.

    Args:
        cfg: a detectron2 CfgNode

    Returns:
        backbone (Backbone): backbone module, must be a subclass of :class:`Backbone`.
    """
    bottom_up = build_VIT_backbone(cfg)
    in_features = cfg.MODEL.FPN.IN_FEATURES
    out_channels = cfg.MODEL.FPN.OUT_CHANNELS
    backbone = FPN(
        bottom_up=bottom_up,
        in_features=in_features,
        out_channels=out_channels,
        norm=cfg.MODEL.FPN.NORM,
        top_block=LastLevelMaxPool(),
        fuse_type=cfg.MODEL.FPN.FUSE_TYPE,
    )
    return backbone



from flask import jsonify


class ResponseCode:
    SUCCESS = 200
    PARAM_WARING = 400
    MESSAGE = "success"


def generate_response(data=None, code=ResponseCode.SUCCESS, msg=ResponseCode.MESSAGE, **kwargs):
    """
    自定义响应
    :param code:状态码
    :param data:返回数据
    :param msg:返回消息
    :param kwargs:
    :return:
    """
    msg = msg or 'success' if code == 200 else msg or 'fail'
    success = True if code == 200 else False
    res = jsonify(dict(code=code, success=success, data=data, msg=msg, **kwargs))
    res.status_code = 200
    return res



'''
Reference: https://huggingface.co/datasets/pierresi/cord/blob/main/cord.py
'''


import json
import os
from pathlib import Path
import datasets
from .image_utils import load_image, normalize_bbox
logger = datasets.logging.get_logger(__name__)
_CITATION = """\
@article{park2019cord,
  title={CORD: A Consolidated Receipt Dataset for Post-OCR Parsing},
  author={Park, Seunghyun and Shin, Seung and Lee, Bado and Lee, Junyeop and Surh, Jaeheung and Seo, Minjoon and Lee, Hwalsuk}
  booktitle={Document Intelligence Workshop at Neural Information Processing Systems}
  year={2019}
}
"""
_DESCRIPTION = """\
https://github.com/clovaai/cord/
"""

def quad_to_box(quad):
    # test 87 is wrongly annotated
    box = (
        max(0, quad["x1"]),
        max(0, quad["y1"]),
        quad["x3"],
        quad["y3"]
    )
    if box[3] < box[1]:
        bbox = list(box)
        tmp = bbox[3]
        bbox[3] = bbox[1]
        bbox[1] = tmp
        box = tuple(bbox)
    if box[2] < box[0]:
        bbox = list(box)
        tmp = bbox[2]
        bbox[2] = bbox[0]
        bbox[0] = tmp
        box = tuple(bbox)
    return box

def _get_drive_url(url):
    base_url = 'https://drive.google.com/uc?id='
    split_url = url.split('/')
    return base_url + split_url[5]

_URLS = [
    _get_drive_url("https://drive.google.com/file/d/1MqhTbcj-AHXOqYoeoh12aRUwIprzTJYI/"),
    _get_drive_url("https://drive.google.com/file/d/1wYdp5nC9LnHQZ2FcmOoC0eClyWvcuARU/")
    # If you failed to download the dataset through the automatic downloader,
    # you can download it manually and modify the code to get the local dataset.
    # Or you can use the following links. Please follow the original LICENSE of CORD for usage.
    # "https://layoutlm.blob.core.windows.net/cord/CORD-1k-001.zip",
    # "https://layoutlm.blob.core.windows.net/cord/CORD-1k-002.zip"
]

class CordConfig(datasets.BuilderConfig):
    """BuilderConfig for CORD"""
    def __init__(self, **kwargs):
        """BuilderConfig for CORD.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(CordConfig, self).__init__(**kwargs)

class Cord(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        CordConfig(name="cord", version=datasets.Version("1.0.0"), description="CORD dataset"),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "words": datasets.Sequence(datasets.Value("string")),
                    "bboxes": datasets.Sequence(datasets.Sequence(datasets.Value("int64"))),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=["O","B-MENU.NM","B-MENU.NUM","B-MENU.UNITPRICE","B-MENU.CNT","B-MENU.DISCOUNTPRICE","B-MENU.PRICE","B-MENU.ITEMSUBTOTAL","B-MENU.VATYN","B-MENU.ETC","B-MENU.SUB_NM","B-MENU.SUB_UNITPRICE","B-MENU.SUB_CNT","B-MENU.SUB_PRICE","B-MENU.SUB_ETC","B-VOID_MENU.NM","B-VOID_MENU.PRICE","B-SUB_TOTAL.SUBTOTAL_PRICE","B-SUB_TOTAL.DISCOUNT_PRICE","B-SUB_TOTAL.SERVICE_PRICE","B-SUB_TOTAL.OTHERSVC_PRICE","B-SUB_TOTAL.TAX_PRICE","B-SUB_TOTAL.ETC","B-TOTAL.TOTAL_PRICE","B-TOTAL.TOTAL_ETC","B-TOTAL.CASHPRICE","B-TOTAL.CHANGEPRICE","B-TOTAL.CREDITCARDPRICE","B-TOTAL.EMONEYPRICE","B-TOTAL.MENUTYPE_CNT","B-TOTAL.MENUQTY_CNT","I-MENU.NM","I-MENU.NUM","I-MENU.UNITPRICE","I-MENU.CNT","I-MENU.DISCOUNTPRICE","I-MENU.PRICE","I-MENU.ITEMSUBTOTAL","I-MENU.VATYN","I-MENU.ETC","I-MENU.SUB_NM","I-MENU.SUB_UNITPRICE","I-MENU.SUB_CNT","I-MENU.SUB_PRICE","I-MENU.SUB_ETC","I-VOID_MENU.NM","I-VOID_MENU.PRICE","I-SUB_TOTAL.SUBTOTAL_PRICE","I-SUB_TOTAL.DISCOUNT_PRICE","I-SUB_TOTAL.SERVICE_PRICE","I-SUB_TOTAL.OTHERSVC_PRICE","I-SUB_TOTAL.TAX_PRICE","I-SUB_TOTAL.ETC","I-TOTAL.TOTAL_PRICE","I-TOTAL.TOTAL_ETC","I-TOTAL.CASHPRICE","I-TOTAL.CHANGEPRICE","I-TOTAL.CREDITCARDPRICE","I-TOTAL.EMONEYPRICE","I-TOTAL.MENUTYPE_CNT","I-TOTAL.MENUQTY_CNT"]
                        )
                    ),
                    "image": datasets.Array3D(shape=(3, 224, 224), dtype="uint8"),
                    "image_path": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            citation=_CITATION,
            homepage="https://github.com/clovaai/cord/",
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        """Uses local files located with data_dir"""
        downloaded_file = dl_manager.download_and_extract(_URLS)
        # move files from the second URL together with files from the first one.
        dest = Path(downloaded_file[0])/"CORD"
        for split in ["train", "dev", "test"]:
            for file_type in ["image", "json"]:
                if split == "test" and file_type == "json":
                    continue
                files = (Path(downloaded_file[1])/"CORD"/split/file_type).iterdir()
                for f in files:
                    os.rename(f, dest/split/file_type/f.name)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": dest/"train"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"filepath": dest/"dev"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"filepath": dest/"test"}
            ),
        ]

    def get_line_bbox(self, bboxs):
        x = [bboxs[i][j] for i in range(len(bboxs)) for j in range(0, len(bboxs[i]), 2)]
        y = [bboxs[i][j] for i in range(len(bboxs)) for j in range(1, len(bboxs[i]), 2)]

        x0, y0, x1, y1 = min(x), min(y), max(x), max(y)

        assert x1 >= x0 and y1 >= y0
        bbox = [[x0, y0, x1, y1] for _ in range(len(bboxs))]
        return bbox

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        ann_dir = os.path.join(filepath, "json")
        img_dir = os.path.join(filepath, "image")
        for guid, file in enumerate(sorted(os.listdir(ann_dir))):
            words = []
            bboxes = []
            ner_tags = []
            file_path = os.path.join(ann_dir, file)
            with open(file_path, "r", encoding="utf8") as f:
                data = json.load(f)
            image_path = os.path.join(img_dir, file)
            image_path = image_path.replace("json", "png")
            image, size = load_image(image_path)
            for item in data["valid_line"]:
                cur_line_bboxes = []
                line_words, label = item["words"], item["category"]
                line_words = [w for w in line_words if w["text"].strip() != ""]
                if len(line_words) == 0:
                    continue
                if label == "other":
                    for w in line_words:
                        words.append(w["text"])
                        ner_tags.append("O")
                        cur_line_bboxes.append(normalize_bbox(quad_to_box(w["quad"]), size))
                else:
                    words.append(line_words[0]["text"])
                    ner_tags.append("B-" + label.upper())
                    cur_line_bboxes.append(normalize_bbox(quad_to_box(line_words[0]["quad"]), size))
                    for w in line_words[1:]:
                        words.append(w["text"])
                        ner_tags.append("I-" + label.upper())
                        cur_line_bboxes.append(normalize_bbox(quad_to_box(w["quad"]), size))
                # by default: --segment_level_layout 1
                # if do not want to use segment_level_layout, comment the following line
                cur_line_bboxes = self.get_line_bbox(cur_line_bboxes)
                bboxes.extend(cur_line_bboxes)
            # yield guid, {"id": str(guid), "words": words, "bboxes": bboxes, "ner_tags": ner_tags, "image": image}
            yield guid, {"id": str(guid), "words": words, "bboxes": bboxes, "ner_tags": ner_tags,
                         "image": image, "image_path": image_path}



import os

from magic_pdf.libs.MakeContentConfig import DropMode, MakeMode
from magic_pdf.pipe.UNIPipe import UNIPipe
from magic_pdf.pipe.OCRPipe import OCRPipe
from magic_doc.conv.base import BaseConv

from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_doc.progress.pupdator import ConvProgressUpdator
from magic_doc.utils import get_repo_directory
from magic_doc.utils.null_writer import NullWriter
from magic_pdf.dict2md.ocr_mkcontent import union_make
from magic_pdf.libs.json_compressor import JsonCompressor
from magic_pdf.rw.AbsReaderWriter import AbsReaderWriter


NULL_IMG_DIR = "/tmp"

class SingletonModelWrapper:

    def __new__(cls):
        if not hasattr(cls, "instance"):
            from magic_doc.model.doc_analysis import DocAnalysis
            apply_ocr = os.getenv("APPLY_OCR", "TRUE") == "TRUE" 
            apply_layout = os.getenv("APPLY_LAYOUT", "TRUE") == "TRUE" 
            apply_formula = os.getenv("APPLY_FORMULA", "FALSE") == "TRUE"
            
            cls.instance = super(SingletonModelWrapper, cls).__new__(cls)
            cls.instance.doc_analysis = DocAnalysis(  # type: ignore
                configs=os.path.join(
                    get_repo_directory(), "resources/model/model_configs.yaml"
                ),
                apply_ocr=apply_ocr, apply_layout=apply_layout, apply_formula=apply_formula,
            )
        return cls.instance
    
    def __call__(self, bytes: bytes):
        from magic_doc.model.doc_analysis import load_images_from_pdf
        images = load_images_from_pdf(bytes, dpi=200)
        return self.doc_analysis(images) # type: ignore


class Pdf(BaseConv):
    def to_md(self, bits: bytes | str, pupdator: ConvProgressUpdator) -> str:
        model_proc = SingletonModelWrapper()
        pupdator.update(0)

        model_list = model_proc(bits)  # type: ignore
        pupdator.update(50)
        jso_useful_key = {
            "_pdf_type": "",
            "model_list": model_list,
        }
        image_writer = NullWriter()
        pipe = OCRPipe(bits, model_list, image_writer, is_debug=True)  # type: ignore
        # pipe.pipe_classify() # 默认ocrpipe的时候不需要再做分类，可以节省时间
        pipe.pipe_parse()
        pupdator.update(100)

        pdf_mid_data = JsonCompressor.decompress_json(pipe.get_compress_pdf_mid_data())
        pdf_info_list = pdf_mid_data["pdf_info"]
        md_content = union_make(pdf_info_list, MakeMode.NLP_MD, DropMode.NONE, NULL_IMG_DIR)
        return md_content # type: ignore

    def to_mid_result(self, image_writer: AbsReaderWriter, bits: bytes | str, pupdator: ConvProgressUpdator) -> list[dict] | dict:
        model_proc = SingletonModelWrapper()
        pupdator.update(0)

        model_list = model_proc(bits)  # type: ignore
        pupdator.update(50)
        jso_useful_key = {
            "_pdf_type": "",
            "model_list": model_list,
        }

        pipe = OCRPipe(bits, model_list, image_writer, is_debug=True)  # type: ignore
        pipe.pipe_classify()
        pipe.pipe_parse()
        pupdator.update(100)

        pdf_mid_data = JsonCompressor.decompress_json(pipe.get_compress_pdf_mid_data())
        pdf_info_list = pdf_mid_data["pdf_info"]
        return pdf_info_list

if __name__ == "__main__":
    with open("/opt/data/pdf/20240423/pdf_test2/ol006018w.pdf", "rb") as f:
        bits_data = f.read()
        parser = Pdf()
        md_content = parser.to_md(
            bits_data, FileBaseProgressUpdator("debug/progress.txt")
        )
        with open("debug/pdf2md.by_model.md", "w") as f:
            f.write(md_content) # type: ignore





from magic_doc.model.sub_modules.self_modify import ModifiedPaddleOCR
from magic_doc.utils import split_to_chunks
import paddle 
from concurrent.futures import ThreadPoolExecutor, as_completed


class SeqOCR:
    def __init__(self, **kwargs):
        self.model = ModifiedPaddleOCR(show_log=False, **kwargs)

    def __call__(self, params):
        """
        params: list[(idx, image, *args)]
        """
        results = [] 
        for idx, cropped_image, single_page_mfdetrec_res in params:
                ocr_res = self.model.ocr(cropped_image, mfd_res=single_page_mfdetrec_res)[0]
                if ocr_res:
                    results.append((idx, ocr_res))

        return results 




import time
from pathlib import Path
from loguru import logger


def upload_image_to_oss(oss_client, file_name, img_path, NULL_IMG_DIR, bucket_name):
    img_object_name = f"pdf/{file_name}/{Path(img_path).name}"
    local_img_path = f"{NULL_IMG_DIR}/images/{Path(img_path).name}"
    t3 = time.time()
    oss_rep = oss_client.put_file(bucket_name, img_object_name, local_img_path)
    t4 = time.time()
    logger.info(f"upload img:{t4 - t3}")
    file_link = oss_rep["file_link"]
    return str(img_path), file_link


def upload_md_to_oss(oss_client, bucket_name, md_object_name, md_content):
    t3 = time.time()
    oss_rep = oss_client.pub_object(bucket_name, md_object_name, md_content)
    t4 = time.time()
    logger.info(f"upload md:{t4 - t3}")
    md_link = oss_rep["file_link"]
    return md_link




# coding=utf-8
'''
Reference: https://huggingface.co/datasets/nielsr/funsd/blob/main/funsd.py
'''
import json
import os

import datasets

from .image_utils import load_image, normalize_bbox


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@article{Jaume2019FUNSDAD,
  title={FUNSD: A Dataset for Form Understanding in Noisy Scanned Documents},
  author={Guillaume Jaume and H. K. Ekenel and J. Thiran},
  journal={2019 International Conference on Document Analysis and Recognition Workshops (ICDARW)},
  year={2019},
  volume={2},
  pages={1-6}
}
"""

_DESCRIPTION = """\
https://guillaumejaume.github.io/FUNSD/
"""


class FunsdConfig(datasets.BuilderConfig):
    """BuilderConfig for FUNSD"""

    def __init__(self, **kwargs):
        """BuilderConfig for FUNSD.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(FunsdConfig, self).__init__(**kwargs)


class Funsd(datasets.GeneratorBasedBuilder):
    """Conll2003 dataset."""

    BUILDER_CONFIGS = [
        FunsdConfig(name="funsd", version=datasets.Version("1.0.0"), description="FUNSD dataset"),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "bboxes": datasets.Sequence(datasets.Sequence(datasets.Value("int64"))),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=["O", "B-HEADER", "I-HEADER", "B-QUESTION", "I-QUESTION", "B-ANSWER", "I-ANSWER"]
                        )
                    ),
                    "image": datasets.Array3D(shape=(3, 224, 224), dtype="uint8"),
                    "image_path": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://guillaumejaume.github.io/FUNSD/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_file = dl_manager.download_and_extract("https://guillaumejaume.github.io/FUNSD/dataset.zip")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": f"{downloaded_file}/dataset/training_data/"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"filepath": f"{downloaded_file}/dataset/testing_data/"}
            ),
        ]

    def get_line_bbox(self, bboxs):
        x = [bboxs[i][j] for i in range(len(bboxs)) for j in range(0, len(bboxs[i]), 2)]
        y = [bboxs[i][j] for i in range(len(bboxs)) for j in range(1, len(bboxs[i]), 2)]

        x0, y0, x1, y1 = min(x), min(y), max(x), max(y)

        assert x1 >= x0 and y1 >= y0
        bbox = [[x0, y0, x1, y1] for _ in range(len(bboxs))]
        return bbox

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        ann_dir = os.path.join(filepath, "annotations")
        img_dir = os.path.join(filepath, "images")
        for guid, file in enumerate(sorted(os.listdir(ann_dir))):
            tokens = []
            bboxes = []
            ner_tags = []

            file_path = os.path.join(ann_dir, file)
            with open(file_path, "r", encoding="utf8") as f:
                data = json.load(f)
            image_path = os.path.join(img_dir, file)
            image_path = image_path.replace("json", "png")
            image, size = load_image(image_path)
            for item in data["form"]:
                cur_line_bboxes = []
                words, label = item["words"], item["label"]
                words = [w for w in words if w["text"].strip() != ""]
                if len(words) == 0:
                    continue
                if label == "other":
                    for w in words:
                        tokens.append(w["text"])
                        ner_tags.append("O")
                        cur_line_bboxes.append(normalize_bbox(w["box"], size))
                else:
                    tokens.append(words[0]["text"])
                    ner_tags.append("B-" + label.upper())
                    cur_line_bboxes.append(normalize_bbox(words[0]["box"], size))
                    for w in words[1:]:
                        tokens.append(w["text"])
                        ner_tags.append("I-" + label.upper())
                        cur_line_bboxes.append(normalize_bbox(w["box"], size))
                # by default: --segment_level_layout 1
                # if do not want to use segment_level_layout, comment the following line
                cur_line_bboxes = self.get_line_bbox(cur_line_bboxes)
                # box = normalize_bbox(item["box"], size)
                # cur_line_bboxes = [box for _ in range(len(words))]
                bboxes.extend(cur_line_bboxes)
            yield guid, {"id": str(guid), "tokens": tokens, "bboxes": bboxes, "ner_tags": ner_tags,
                         "image": image, "image_path": image_path}


import os

from s3pathlib import S3Path

from magic_doc.utils.config import read_config


def get_local_dir():
    config = read_config()
    return config.get("temp-output-dir", "/tmp")


def prepare_env(doc_file_name, doc_type="") -> str:
    if doc_type == "":
        doc_type = "unknown"
    local_parent_dir = os.path.join(
        get_local_dir(), "magic-doc", doc_type, doc_file_name
    )

    # local_image_dir = os.path.join(local_parent_dir, "images")
    local_md_dir = local_parent_dir
    # os.makedirs(local_image_dir, exist_ok=True)
    os.makedirs(local_md_dir, exist_ok=True)
    return str(local_md_dir)


def remove_non_official_s3_args(s3path):
    """
    example: s3://abc/xxxx.json?bytes=0,81350 ==> s3://abc/xxxx.json
    """
    arr = s3path.split("?")
    return arr[0]


def parse_s3path(s3path: str):
    p = S3Path(remove_non_official_s3_args(s3path))
    return p.bucket, p.key



def singleton_func(cls):
    instance = {}

    def _singleton(*args, **kwargs):
        if cls not in instance:
            instance[cls] = cls(*args, **kwargs)
        return instance[cls]

    return _singleton



from flask import Flask
from flask_restful import Api
from flask_cors import CORS
from flask_sqlalchemy import SQLAlchemy as _SQLAlchemy
from flask_migrate import Migrate
from contextlib import contextmanager
from flask_jwt_extended import JWTManager
from flask_marshmallow import Marshmallow


class SQLAlchemy(_SQLAlchemy):
    @contextmanager
    def auto_commit(self):
        try:
            yield
            db.session.commit()
            db.session.flush()
        except Exception as e:
            db.session.rollback()
            raise e


app = Flask(__name__)
CORS(app, supports_credentials=True)
db = SQLAlchemy()
migrate = Migrate()
jwt = JWTManager()
ma = Marshmallow()


from flask import Blueprint
from .magic_pdf_view import *
from .magic_html_view import *
from ..extentions import Api

analysis_blue = Blueprint('analysis', __name__, url_prefix='/analysis')

api = Api(analysis_blue)
api.add_resource(MagicPdfView, '/pdf')
api.add_resource(MagicHtmlView, '/html')


import sys
from pathlib import Path

base_dir = Path(__file__).resolve().parent
root_dir = base_dir.parent.parent
sys.path.append(str(root_dir))

from api import create_app
import yaml

config_path = base_dir / "config/config.yaml"


class ConfigMap(dict):
    __setattr__ = dict.__setitem__
    __getattr__ = dict.__getitem__


with open(str(config_path), mode='r', encoding='utf-8') as fd:
    data = yaml.load(fd, Loader=yaml.FullLoader)
    _config = data.get(data.get("CurrentConfig", "DevelopmentConfig"))
config = ConfigMap()
for k, v in _config.items():
    config[k] = v
config['base_dir'] = base_dir
database = _config.get("database")
if database:
    if database.get("type") == "sqlite":
        database_uri = f'sqlite:///{base_dir}/{database.get("path")}'
    elif database.get("type") == "mysql":
        database_uri = f'mysql+pymysql://{database.get("user")}:{database.get("password")}@{database.get("host")}:{database.get("port")}/{database.get("database")}?'
    else:
        database_uri = ''
    config['SQLALCHEMY_DATABASE_URI'] = database_uri
app = create_app(config)

if __name__ == '__main__':
    app.run(host="0.0.0.0", port=5556, debug=True)


import oss2
from magic_doc.restful_api.common.ext import singleton_func
from loguru import logger


@singleton_func
class Oss(object):
    def __init__(self, access_key_id, access_secret_key, bucket_name, endpoint, expires=60):
        self.access_key_id = access_key_id
        self.access_secret_key = access_secret_key
        self.bucket_name = bucket_name
        self.endpoint = endpoint
        self.expires = expires
        self.auth = oss2.Auth(self.access_key_id, self.access_secret_key)

    def create_bucket(self, bucket_name=None):
        """
        创建存储空间
        :param bucket_name:  bucket名称
        :return:
        """
        bucket = oss2.Bucket(self.auth, self.endpoint, bucket_name if bucket_name else self.bucket_name)
        # 设置存储空间为私有读写权限
        # bucket.create_bucket(oss2.models.BUCKET_ACL_PRIVATE)
        bucket.create_bucket()
        return True

    def delete_bucket(self, bucket_name=None):
        """
        删除存储空间
        :param bucket_name:  bucket名称
        :return:
        """
        bucket = oss2.Bucket(self.auth, self.endpoint, bucket_name if bucket_name else self.bucket_name)
        try:
            bucket.delete_bucket()
            return True
        except oss2.exceptions.BucketNotEmpty:
            logger.error('bucket is not empty.')
            return False
        except oss2.exceptions.NoSuchBucket:
            logger.error('bucket does not exist')
            return False

    def pub_object(self, bucket_name=None, object_name=None, object_data=None):
        """
        上传文件
            Str
            Bytes
            Unicode
            Stream
        :param bucket_name:  bucket名称
        :param object_name:  不包含Bucket名称组成的Object完整路径
        :param object_data:
        :return:
        """
        bucket = oss2.Bucket(self.auth, self.endpoint, bucket_name if bucket_name else self.bucket_name)
        result = bucket.put_object(object_name, object_data)
        file_link = bucket.sign_url('GET', object_name, self.expires, slash_safe=True)
        return {
            "status": result.status,
            "request_id": result.request_id,
            "etag": result.etag,
            "date": result.headers['date'],
            "file_link": file_link
        }

    def put_file(self, bucket_name=None, object_name=None, file_path=None):
        """
        上传文件
            file
        :param bucket_name:  bucket名称
        :param object_name:  不包含Bucket名称组成的Object完整路径
        :param file_path:   文件路径
        :return:
        """
        bucket = oss2.Bucket(self.auth, self.endpoint, bucket_name if bucket_name else self.bucket_name)
        result = bucket.put_object_from_file(object_name, file_path)
        file_link = bucket.sign_url('GET', object_name, self.expires, slash_safe=True)
        return {
            "status": result.status,
            "request_id": result.request_id,
            "etag": result.etag,
            "date": result.headers['date'],
            "file_link": file_link
        }

    def delete_objects(self, bucket_name=None, object_name=None):
        """
        批量删除文件
        :param bucket_name:  bucket名称
        :param object_name:  不包含Bucket名称组成的Object完整路径列表
        :return:
        """
        if object_name is None:
            object_name = []
        bucket = oss2.Bucket(self.auth, self.endpoint, bucket_name if bucket_name else self.bucket_name)
        result = bucket.batch_delete_objects(object_name)
        return result.deleted_keys

    def download_file(self, bucket_name=None, object_name=None, save_path=None):
        """
        下载文件到本地
        :param bucket_name:  bucket名称
        :param object_name:  不包含Bucket名称组成的Object完整路径
        :param save_path:  保存路径
        :return:
        """
        bucket = oss2.Bucket(self.auth, self.endpoint, bucket_name if bucket_name else self.bucket_name)
        bucket.get_object_to_file(object_name, save_path)



import os
import traceback
from datetime import datetime
import click
from pathlib import Path
from magic_doc.libs.version import __version__

from magic_doc.utils.config import get_s3_config
from magic_doc.utils.path_utils import get_local_dir, parse_s3path, prepare_env
from magic_doc.docconv import DocConverter, S3Config
from loguru import logger

log_level = "ERROR"
log_dir = os.path.join(get_local_dir(), "magic-doc", "logs")
if not Path(log_dir).exists():
    Path(log_dir).mkdir(parents=True, exist_ok=True)
log_name = f'log_{datetime.now().strftime("%Y-%m-%d")}.log'
log_file_path = os.path.join(log_dir, log_name)
logger.add(str(log_file_path), rotation='00:00', encoding='utf-8', level=log_level, enqueue=True)


def abort(message=None, exit_code=1):
    click.echo(click.style(message, fg='red'))
    exit(exit_code)


total_cost_time = 0
total_convert_error = 0
total_file_broken = 0
total_unsupported_files = 0
total_time_out = 0
total_success_convert = 0


@click.command()
@click.version_option(__version__, "--version", "-v", help="显示版本信息")
@click.option('-f', '--file-path', 'input_file_path', type=click.STRING,
              help='file path, support s3/local/list, list file need end with ".list"')
@click.option('-p', '--progress-file-path', 'progress_file_path', default="", type=click.STRING,
              help='path to the progress file to save')
@click.option('-t', '--conv-timeout', 'conv_timeout', default=60, type=click.INT, help='timeout')
def cli_conv(input_file_path, progress_file_path, conv_timeout=None):
    global total_cost_time, total_convert_error, total_file_broken, \
        total_unsupported_files, total_time_out, total_success_convert

    def parse_doc(doc_path, pf_path=None):
        """使用全局变量统计耗时和error数量"""
        global total_cost_time, total_convert_error, total_file_broken, \
            total_unsupported_files, total_time_out, total_success_convert
        try:
            file_name = str(Path(doc_path).stem)
            '''创建同名进度缓存文件'''
            if not pf_path:
                pf_path = f"/tmp/{file_name}.txt"
            '''如果文档路径为s3链接，先获取s3配置并初始化'''
            if doc_path.startswith("s3://"):
                bucket, key = parse_s3path(doc_path)
                ak, sk, endpoint = get_s3_config(bucket)
                s3_config = S3Config(ak, sk, endpoint)
            else:
                '''非s3路径不需要初始化s3配置'''
                s3_config = None
            doc_conv = DocConverter(s3_config)
            markdown_string, cost_time = doc_conv.convert(doc_path, pf_path, conv_timeout)
            total_cost_time += cost_time
            logger.info(f"convert {doc_path} to markdown, cost {cost_time} seconds")
            # click.echo(markdown_string)
            base_name, doc_type = os.path.splitext(doc_path)
            out_put_dir = prepare_env(file_name, doc_type.lstrip(".").lower())
            with open(os.path.join(out_put_dir, file_name + ".md"), "w", encoding='utf-8') as md_file:
                md_file.write(markdown_string)
            total_success_convert += 1
            return cost_time
        except Exception as e:
            logger.exception(e)
            # 只统计转换出错的数量
            if "Convert failed" in str(e):
                total_convert_error += 1
            elif "Convert timeout" in str(e):
                total_time_out += 1
            elif "File is broken" in str(e):
                total_file_broken += 1
            elif "Unsupported file format" in str(e):
                total_unsupported_files += 1
            # abort(f'Error: {traceback.format_exc()}')

    if not input_file_path:
        logger.error(f"Error: Missing argument '--file-path'.")
        abort(f"Error: Missing argument '--file-path'.")
    else:
        '''适配多个文档的list文件输入'''
        if input_file_path.endswith(".list"):
            with open(input_file_path, "r") as f:
                for line in f.readlines():
                    line = line.strip()
                    parse_doc(line, progress_file_path)
        else:
            '''适配单个文档的输入'''
            parse_doc(input_file_path, progress_file_path)

    logger.info(f"total cost time: {int(total_cost_time)} seconds")
    logger.info(f"total convert error: {total_convert_error}")
    logger.info(f"total file broken: {total_file_broken}")
    logger.info(f"total unsupported files: {total_unsupported_files}")
    logger.info(f"total time out: {total_time_out}")
    logger.info(f"total success: {total_success_convert}")


if __name__ == '__main__':
    cli_conv()



import io
import os
import requests


from pathlib import Path
from typing import IO, List
from zipfile import ZipFile
from loguru import logger

from werkzeug.datastructures import FileStorage

from magic_doc.contrib.office import OfficeExtractor
from magic_doc.contrib.model import ExtractResponse, Content, Page
from pedia_document_parser.config import Config

ext_map: dict[str, str] = {
    ".ppt": "application/vnd.ms-powerpoint",
    ".pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    ".doc": "application/msword",
    ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
}


def extract_office_to_dir(
    file_bytes: IO[bytes],
    ext: str,
    dir: Path,
    media_dir: Path,
    skip_image: bool,
):
    config = Config()
    data = file_bytes.read()

    if skip_image:
        headers = {
            "Content-type": ext_map[ext],
            "Accept": "application/json",
            "X-Tika-Skip-Embedded": "true",
        }
        response = requests.put(
            f"{config.tika}/tika/text",
            headers=headers,
            data=data,
        )
        content: str = response.json()["X-TIKA:content"]

        with dir.joinpath("__TEXT__").open("w") as f:
            f.write(content)

        return

    headers = {
        "Content-type": ext_map[ext],
        "Accept": "application/zip",
    }
    response = requests.put(
        f"{config.tika}/unpack/all",
        headers=headers,
        data=data,
    )

    if not response.ok:
        raise Exception(f"tika failed {response.text}")

    with ZipFile(io.BytesIO(response.content), "r") as zip:
        for zipped_file in zip.namelist():
            with zip.open(zipped_file, "r") as f:
                if zipped_file == "__TEXT__" or zipped_file == "__METADATA__":
                    dir.joinpath(zipped_file).parent.mkdir(parents=True, exist_ok=True)
                    with open(dir.joinpath(zipped_file), "wb") as w:
                        w.write(f.read())
                else:
                    media_dir.joinpath(zipped_file).parent.mkdir(
                        parents=True,
                        exist_ok=True,
                    )
                    with open(media_dir.joinpath(zipped_file), "wb") as w:
                        w.write(f.read())
            logger.debug(f"save file {zipped_file}")


def list_files_recursive(root: Path) -> List[Path]:
    result = []
    for entry in os.listdir(root):
        full_path = root.joinpath(entry)
        if os.path.isdir(full_path):
            result.extend(list_files_recursive(full_path))
        else:
            result.append(full_path)
    return result


class GeneralOfficeExtractor(OfficeExtractor):
    def __init__(self) -> None:
        super().__init__()

    def setup(self):
        pass

    def extract(
        self,
        r: FileStorage | Path,
        id: str,
        dir: Path,
        media_dir: Path,
        skip_image: bool,
    ) -> ExtractResponse:
        if type(r) is FileStorage:
            r.name = os.path.basename(r.filename)

        ext: str = os.path.splitext(r.name)[1]
        ext = ext.lower()

        if ext not in ext_map:
            raise Exception(f"{ext} not supported")

        if type(r) is FileStorage:
            extract_office_to_dir(r.stream, ext, dir, media_dir, skip_image)
        elif issubclass(type(r), Path):
            extract_office_to_dir(open(r, "rb"), ext, dir, media_dir, skip_image)
        else:
            raise Exception(type(r))

        page = Page(page_no=0, content_list=[])

        media_map: dict[Path, str] = {}
        for media_file in list_files_recursive(media_dir):
            media_map[media_file] = self.generate_img_path(id, media_file.name)

        match ext:
            case ".pptx" | ".ppt":
                with open(dir.joinpath("__TEXT__"), "r") as f:
                    text: str = f.read()

                content_list = [Content(type="text", data=text)]
                page["content_list"] = content_list

                self.upload_background(id, media_map)

                return [page]
                # texts: list[str] = text.split("\n")
                # new_texts: list[str] = []

                # enters = 0
                # for text in texts:
                #     if len(text.strip()) == 0:
                #         enters += 1
                #         if enters <= 3:
                #             new_texts.append("")
                #     else:
                #         enters = 0
                #         new_texts.append(text)

                # texts = "\n".join(new_texts).replace("\n\n\n","\npage_break\n").split("\n")

                # pages = [Page(page_no=0, content_list=[])]
                # for text in texts:
                #     if text == "page_break":
                #         page = Page(page_no=len(pages), content_list=[])
                #         pages.append(page)
                #     else:
                #         pages[len(pages) - 1]["content_list"].append(
                #             Content(type="text", data=text)
                #         )

                # page = Page(page_no=len(pages), content_list=[])

                # img_map = {}
                # for pic_file in media_dir.glob("*"):
                #     img_map[pic_file] = self.generate_img_path(id, pic_file.name)
                #     page["content_list"].append(
                #         Content(type="image", data=img_map[pic_file])
                #     )

                # pages.append(page)
                # self.upload_background(id, media_map)

                # return pages
            case ".doc" | ".docx":
                with open(dir.joinpath("__TEXT__"), "r") as f:
                    text = f.read()

                content_list = [Content(type="text", data=text)]

                for media_file, s3_path in media_map.items():
                    new_content_list = []

                    for content in content_list:
                        if content["type"] != "text":
                            new_content_list.append(content)
                            continue

                        pos = content["data"].find(media_file.name)
                        if pos == -1:
                            new_content_list.append(content)
                            continue

                        lpos = content["data"][:pos].rindex("[")

                        assert lpos != -1

                        r_text = content["data"][pos + len(media_file.name) + 1 :]
                        l_text = content["data"][:lpos]

                        new_content_list.append(Content(type="text", data=l_text))
                        new_content_list.append(Content(type="image", data=s3_path))
                        new_content_list.append(Content(type="text", data=r_text))

                    content_list = new_content_list

                page["content_list"] = content_list

                self.upload_background(id, media_map)

                return [page]


if __name__ == "__main__":
    e = GeneralOfficeExtractor()
    files = [
        # "/home/SENSETIME/wuziming/doc/doc/【英文-课件】MIT15_082JF10_lec10.3MB.ppt",
        # "/home/SENSETIME/wuziming/doc/doc/【英文-课件】MIT15_082JF10_lec17.8MB.ppt",
        # "/home/SENSETIME/wuziming/doc/doc/【英文-课件】MIT15_082JF10_lec18.4MB.ppt",
        "/home/SENSETIME/wuziming/doc/doc/【中简】商业项目市场分析与产品定位报告.ppt",
        # "/home/SENSETIME/wuziming/doc/doc/【英文-习题】MIT_prs_w01d1.doc",
        # "/home/SENSETIME/wuziming/doc/doc/【英文-习题】MIT_prs_w05d2.doc",
        # "/home/SENSETIME/wuziming/doc/doc/【中简】模电自测第四版.doc",
    ]

    import uuid

    for f in files:
        id = uuid.uuid4().hex
        x = e.run(id, Path(f))

        content = ""
        for p in x:
            content += f"\n====== page {p['page_no']} ======\n"
            for pp in p["content_list"]:
                content += pp["data"] + "\n"

        print(content)

    e.wait_all()



import os
from datetime import datetime
from pathlib import Path
from loguru import logger
from .extentions import app, db, migrate, jwt, ma
from magic_doc.restful_api.common.web_hook import before_request

root_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


def init_app_log(config):
    """
    Setup logging
    :param config:  config file
    :return:
    """
    log_path = os.path.join(Path(__file__).parent.parent, "log")
    if not Path(log_path).exists():
        Path(log_path).mkdir(parents=True, exist_ok=True)
    log_level = config.get("LOG_LEVEL")
    log_name = f'log_{datetime.now().strftime("%Y-%m-%d")}.log'
    log_file_path = os.path.join(log_path, log_name)
    logger.add(str(log_file_path), rotation='00:00', encoding='utf-8', level=log_level, enqueue=True)
    return logger


def _register_db(flask_app):
    db.init_app(flask_app)
    with app.app_context():
        db.create_all()


def create_app(config):
    """
    Create and configure an instance of the Flask application
    :param config:
    :return:
    """
    app.static_folder = os.path.join(root_dir, "static")
    if config is None:
        config = {}
    app.config.update(config)
    init_app_log(config)
    # _register_db(app)
    migrate.init_app(app=app, db=db)
    jwt.init_app(app=app)
    ma.init_app(app=app)
    from .analysis import analysis_blue
    app.register_blueprint(analysis_blue)

    app.before_request(before_request)

    return app



import io
import tempfile
import zipfile
import xml.etree.ElementTree as ET
from pathlib import Path

from loguru import logger

from magic_doc.contrib.model import Content, Page
from magic_doc.contrib.office.docx_extract import DocxExtractor
from magic_doc.conv.base import BaseConv
from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_doc.progress.pupdator import ConvProgressUpdator


class Docx(BaseConv):
    def __init__(self):
        super().__init__()

    def to_md(self, bits: bytes, pupdator: ConvProgressUpdator) -> str:
        page_list = self.docx_to_pagelist(bits, pupdator)
        md_content_list = []
        for page in page_list:
            page_content_list = page['content_list']
            total = len(page_content_list)
            for index, content in enumerate(page_content_list):
                progress = 50 + int(index / total * 50)
                pupdator.update(progress)
                if content['type'] == 'image':
                    pass
                elif content['type'] in ["text", "md"]:
                    data = content['data']
                    md_content_list.append(data)
        return "\n".join(md_content_list)

    def docx_to_pagelist(self, bits, pupdator: ConvProgressUpdator) -> list[Page]:
        with tempfile.TemporaryDirectory() as temp_path:
            temp_dir = Path(temp_path)
            media_dir = temp_dir / "media"
            media_dir.mkdir()
            file_path = temp_dir / "tmp.docx"
            file_path.write_bytes(bits)
            docx_extractor = DocxExtractor()
            pages = docx_extractor.extract(file_path, "tmp", temp_dir, media_dir, True)
            pupdator.update(50)
            return pages


if __name__ == '__main__':
    pupdator = FileBaseProgressUpdator("/tmp/p.txt")
    docx = Docx()
    logger.info(docx.to_md(open(r"D:\project\20240514magic_doc\doc_ppt\doc\demo\文本+表+图.docx", "rb").read(), pupdator))



import json
from magic_doc.conv.base import BaseConv
from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_html import GeneralExtractor
from magic_doc.progress.pupdator import ConvProgressUpdator
from loguru import logger

extractor = GeneralExtractor()


class Html(BaseConv):

    def __init__(self):
        super().__init__()

    @logger.catch
    def to_md(self, html: str, pupdator: ConvProgressUpdator, **kwargs) -> str:
        """
        从HTML中提取主体区域内容
        :param html:  html文本
        :param kwargs: 可选参数
            base_url   网页地址
            html_type  网页类型(支持3种)
                1. article  文章类
                2. forum    论坛类
                3. weixin   微信文章
        :return:   {
                    "base_url": "https://example.com/",
                    "drop_list": false,
                    "html": "<div><body...",
                    "title": "",
                    "xp_num": "others"
                  }
        """
        base_url = kwargs.get("base_url", "")
        html_type = kwargs.get("html_type", None)
        data = extractor.extract(html, base_url=base_url, html_type=html_type)
        return json.dumps(data, ensure_ascii=False)


if __name__ == '__main__':
    html_str = "<!doctype html><html><head><title>Example Domain</title><meta charset='utf-8' /><meta http-equiv='Content-type' content='text/html; charset=utf-8' /><meta name='viewport' content='width=device-width, initial-scale=1' /></head><body><div><h1>Example Domain</h1><p>This domain is for use in illustrative examples in documents. You may use this domain in literature without prior coordination or asking for permission.</p><p>This domain is for use in illustrative examples in documents. You may use this domain in literature without prior coordination or asking for permission.</p><p>This domain is for use in illustrative examples in documents. You may use this domain in literature without prior coordination or asking for permission.</p><p><a href='https://www.iana.org/domains/example'>More information...</a></p></div></body></html>"
    pupdator = FileBaseProgressUpdator("/tmp/p.txt")
    result = Html(pupdator).to_md(html=html_str, pupdator=pupdator)
    print(result)



import argparse
import logging
import os
import time

import cv2
import fitz
import numpy as np
import torch
import unimernet.tasks as tasks
import yaml
from PIL import Image
from torch.utils.data import DataLoader, Dataset
from torchvision import transforms
from ultralytics import YOLO
from unimernet.common.config import Config
from unimernet.processors import load_processor

from magic_doc.model.sub_modules.layoutlmv3.model_init import Layoutlmv3_Predictor
from magic_doc.model.sub_modules.post_process import (
    get_croped_image,
    latex_rm_whitespace,
)
from magic_doc.model.parallel_ocr import ParallelOCR
from magic_doc.model.seq_ocr import SeqOCR
from magic_doc.model.parallel_layout import ParallelLayout
from magic_doc.model.seq_layout import SeqLayout

import paddle 

from magic_doc.utils.yaml_load import patch_yaml_load_with_env
from magic_doc.utils import get_repo_directory

logging.disable(logging.WARNING)


class MathDataset(Dataset):
    def __init__(self, image_paths, transform=None):
        self.image_paths = image_paths
        self.transform = transform

    def __len__(self):
        return len(self.image_paths)

    def __getitem__(self, idx):
        # if not pil image, then convert to pil image
        if isinstance(self.image_paths[idx], str):
            raw_image = Image.open(self.image_paths[idx])
        else:
            raw_image = self.image_paths[idx]
        if self.transform:
            image = self.transform(raw_image)
        return image


class DocAnalysis(object):
    """
    Description:
      class definition of DocAnalysis module:
    """

    def __init__(
        self, configs="magic_doc/resources/model/model_configs.yaml", **kwargs
    ):
        """
        Description:
          initialize the class instance

        Parameters:
          configs: path to config that contains model weights.
          apply_layout: do layout analysis or not, must be True (defaults to be True).
          apply_formula: do formulat detection and recognition or not, defaults to be False.
          apply_ocr: do ocr(text detection and recognition) or not, defaults to be False.

        """
        self.configs = patch_yaml_load_with_env(configs, "model", yaml.FullLoader)  # load config and patch with env var !
        self.apply_layout = kwargs.get("apply_layout", self.configs["models"]["layout"])
        self.apply_formula = kwargs.get(
            "apply_formula", self.configs["models"]["formula"]
        )
        self.apply_ocr = kwargs.get("apply_ocr", self.configs["models"]["ocr"])
        logging.info(
            "DocAnalysis init, this may take some times. apply_layout: {}, apply_formula: {}, apply_ocr: {}".format(
                self.apply_layout, self.apply_formula, self.apply_ocr
            )
        )
        self.device = "cuda" if torch.cuda.is_available() else "cpu"
        logging.info("using device: {}".format(self.device))
        assert self.apply_layout, "DocAnalysis must contain layout model."

        if torch.cuda.device_count() > 1:
            self.layout_model = ParallelLayout(self.configs["weights"]["layout"])
        else:
            self.layout_model = SeqLayout(self.configs["weights"]["layout"])

        if self.apply_formula:
            self.mfd_model = YOLO(self.configs["weights"]["mfd"])
            args = argparse.Namespace(
                cfg_path=os.path.join(get_repo_directory(), "resources/model/UniMERNet/demo_old.yaml"),
                options=None,
            )
            cfg = Config(args)
            cfg.config.model.pretrained = os.path.join(
                self.configs["weights"]["mfr"], "pytorch_model.bin"
            )
            cfg.config.model.model_config.model_name = self.configs["weights"]["mfr"]
            cfg.config.model.tokenizer_config.path = self.configs["weights"]["mfr"]
            task = tasks.setup_task(cfg)
            model = task.build_model(cfg)
            self.mfr_model = model.to(self.device)
            mfr_vis_processor = load_processor(
                "formula_image_eval",
                cfg.config.datasets.formula_rec_eval.vis_processor.eval,
            )
            self.mfr_transform = transforms.Compose(
                [
                    mfr_vis_processor,
                ]
            )
        if self.apply_ocr:
            if paddle.device.cuda.device_count() > 1:
                self.ocr_model = ParallelOCR()
            else:
                self.ocr_model = SeqOCR(use_gpu=True)

        logging.info("DocAnalysis init done!")

    def __call__(self, image_list):
        """
        Description:
          do document analysis on input images

        Parameters:
          image_list: list of image array, [np.array, ...]

        Return:
          result: doc analysis result
        """
        doc_layout_result = []
        latex_filling_list = []
        mf_image_list = []

        layout_inference_reqs = [] 

        for idx, image in enumerate(image_list):
            layout_inference_reqs.append((idx, image)) # 

        layout_inference_results = self.layout_model(layout_inference_reqs)

        for idx, layout_res in layout_inference_results:
            image = image_list[idx]
            img_H, img_W = image.shape[0], image.shape[1]

            if self.apply_formula:
                mfd_res = self.mfd_model.predict(
                    image, imgsz=1888, conf=0.25, iou=0.45, verbose=False
                )[0]
                for xyxy, conf, cla in zip(
                    mfd_res.boxes.xyxy.cpu(),
                    mfd_res.boxes.conf.cpu(),
                    mfd_res.boxes.cls.cpu(),
                ):
                    xmin, ymin, xmax, ymax = [int(p.item()) for p in xyxy]
                    new_item = {
                        "category_id": 13 + int(cla.item()),
                        "poly": [xmin, ymin, xmax, ymin, xmax, ymax, xmin, ymax],
                        "score": round(float(conf.item()), 2),
                        "latex": "",
                    }
                    layout_res["layout_dets"].append(new_item)
                    latex_filling_list.append(new_item)
                    bbox_img = get_croped_image(
                        Image.fromarray(image), [xmin, ymin, xmax, ymax]
                    )
                    mf_image_list.append(bbox_img)

            layout_res["page_info"] = dict(page_no=idx, height=img_H, width=img_W)
            doc_layout_result.append(layout_res)
        
        if self.apply_formula:
            # 公式识别，因为识别速度较慢，为了提速，把单个pdf的所有公式裁剪完，一起批量做识别。
            a = time.time()
            dataset = MathDataset(mf_image_list, transform=self.mfr_transform)
            dataloader = DataLoader(dataset, batch_size=128, num_workers=32)
            mfr_res = []
            for imgs in dataloader:
                imgs = imgs.to(self.device)
                output = self.mfr_model.generate({"image": imgs})
                mfr_res.extend(output["pred_str"])
            for res, latex in zip(latex_filling_list, mfr_res):
                res["latex"] = latex_rm_whitespace(latex)
            b = time.time()
            logging.info(
                "formula nums: {}, mfr time: {}".format(
                    len(mf_image_list), round(b - a, 2)
                )
            )

        if self.apply_ocr:
            # ocr识别
            ocr_det_reqs = []

            a = time.time()
            for idx, image in enumerate(image_list):
                pil_img = Image.fromarray(cv2.cvtColor(image, cv2.COLOR_RGB2BGR))
                single_page_res = doc_layout_result[idx]["layout_dets"]
                single_page_mfdetrec_res = []
                if self.apply_formula:
                    for res in single_page_res:
                        if int(res["category_id"]) in [13, 14]:
                            xmin, ymin = int(res["poly"][0]), int(res["poly"][1])
                            xmax, ymax = int(res["poly"][4]), int(res["poly"][5])
                            single_page_mfdetrec_res.append(
                                {
                                    "bbox": [xmin, ymin, xmax, ymax],
                                }
                            )
                for res in single_page_res:
                    if int(res["category_id"]) in [
                        0,
                        1,
                        2,
                        4,
                        6,
                        7,
                    ]:  # 需要进行ocr的类别
                        xmin, ymin = int(res["poly"][0]), int(res["poly"][1])
                        xmax, ymax = int(res["poly"][4]), int(res["poly"][5])
                        crop_box = [xmin, ymin, xmax, ymax]
                        cropped_img = Image.new("RGB", pil_img.size, "white")
                        cropped_img.paste(pil_img.crop(crop_box), crop_box)
                        cropped_img = cv2.cvtColor(
                            np.asarray(cropped_img), cv2.COLOR_RGB2BGR
                        )
                        ocr_det_reqs.append((idx, cropped_img, single_page_mfdetrec_res))

            for idx, ocr_res in self.ocr_model(ocr_det_reqs):
                if ocr_res:
                    for box_ocr_res in ocr_res:
                        p1, p2, p3, p4 = box_ocr_res[0]
                        text, score = box_ocr_res[1]
                        doc_layout_result[idx]["layout_dets"].append(
                            {
                                "category_id": 15,
                                "poly": p1 + p2 + p3 + p4,
                                "score": round(score, 2),
                                "text": text,
                            }
                        )
            b = time.time()
            logging.info("ocr time: {}".format(round(b - a, 2)))
        return doc_layout_result


def load_images_from_pdf(pdf_data_bytes, dpi=72):
    doc = fitz.open("pdf", pdf_data_bytes)
    images = []

    for i in range(len(doc)):
        page = doc[i]
        pix = page.get_pixmap(matrix=fitz.Matrix(dpi / 72, dpi / 72))
        image = Image.frombytes("RGB", [pix.width, pix.height], pix.samples)
        # images.append(image)
        images.append(np.array(image)[:, :, ::-1])
    return images


if __name__ == "__main__":
    doc_proc = DocAnalysis(apply_ocr=True, apply_layout=True, apply_formula=False)
    with open("/opt/data/pdf/20240423/pdf_test2/tc.2013.10.pdf", "rb") as f:
        data = f.read()
        results = doc_proc(load_images_from_pdf(data))
    print(results)



import time
import copy
import logging
import base64
import cv2
import numpy as np
from io import BytesIO
from PIL import Image

from paddleocr import PaddleOCR
from ppocr.utils.logging import get_logger
from ppocr.utils.utility import check_and_read, alpha_to_color, binarize_img
from tools.infer.utility import draw_ocr_box_txt, get_rotate_crop_image, get_minarea_rect_crop
logger = get_logger()

def img_decode(content: bytes):
    np_arr = np.frombuffer(content, dtype=np.uint8)
    return cv2.imdecode(np_arr, cv2.IMREAD_UNCHANGED)

def check_img(img):
    if isinstance(img, bytes):
        img = img_decode(img)
    if isinstance(img, str):
        image_file = img
        img, flag_gif, flag_pdf = check_and_read(image_file)
        if not flag_gif and not flag_pdf:
            with open(image_file, 'rb') as f:
                img_str = f.read()
                img = img_decode(img_str)
            if img is None:
                try:
                    buf = BytesIO()
                    image = BytesIO(img_str)
                    im = Image.open(image)
                    rgb = im.convert('RGB')
                    rgb.save(buf, 'jpeg')
                    buf.seek(0)
                    image_bytes = buf.read()
                    data_base64 = str(base64.b64encode(image_bytes),
                                      encoding="utf-8")
                    image_decode = base64.b64decode(data_base64)
                    img_array = np.frombuffer(image_decode, np.uint8)
                    img = cv2.imdecode(img_array, cv2.IMREAD_COLOR)
                except:
                    logger.error("error in loading image:{}".format(image_file))
                    return None
        if img is None:
            logger.error("error in loading image:{}".format(image_file))
            return None
    if isinstance(img, np.ndarray) and len(img.shape) == 2:
        img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)

    return img

def sorted_boxes(dt_boxes):
    """
    Sort text boxes in order from top to bottom, left to right
    args:
        dt_boxes(array):detected text boxes with shape [4, 2]
    return:
        sorted boxes(array) with shape [4, 2]
    """
    num_boxes = dt_boxes.shape[0]
    sorted_boxes = sorted(dt_boxes, key=lambda x: (x[0][1], x[0][0]))
    _boxes = list(sorted_boxes)

    for i in range(num_boxes - 1):
        for j in range(i, -1, -1):
            if abs(_boxes[j + 1][0][1] - _boxes[j][0][1]) < 10 and \
                    (_boxes[j + 1][0][0] < _boxes[j][0][0]):
                tmp = _boxes[j]
                _boxes[j] = _boxes[j + 1]
                _boxes[j + 1] = tmp
            else:
                break
    return _boxes


def formula_in_text(mf_bbox, text_bbox):
    x1, y1, x2, y2 = mf_bbox
    x3, y3 = text_bbox[0]
    x4, y4 = text_bbox[2]
    left_box, right_box = None, None
    same_line = abs((y1+y2)/2 - (y3+y4)/2) / abs(y4-y3) < 0.2
    if not same_line:
        return False, left_box, right_box
    else:
        drop_origin = False
        left_x = x1 - 1
        right_x = x2 + 1
        if x3 < x1 and x2 < x4:
            drop_origin = True
            left_box = np.array([text_bbox[0], [left_x, text_bbox[1][1]], [left_x, text_bbox[2][1]], text_bbox[3]]).astype('float32')
            right_box = np.array([[right_x, text_bbox[0][1]], text_bbox[1], text_bbox[2], [right_x, text_bbox[3][1]]]).astype('float32')
        if x3 < x1 and x1 <= x4 <= x2:
            drop_origin = True
            left_box = np.array([text_bbox[0], [left_x, text_bbox[1][1]], [left_x, text_bbox[2][1]], text_bbox[3]]).astype('float32')
        if x1 <= x3 <= x2 and x2 < x4:
            drop_origin = True
            right_box = np.array([[right_x, text_bbox[0][1]], text_bbox[1], text_bbox[2], [right_x, text_bbox[3][1]]]).astype('float32')
        if x1 <= x3 < x4 <= x2:
            drop_origin = True
        return drop_origin, left_box, right_box

    
def update_det_boxes(dt_boxes, mfdetrec_res):
    new_dt_boxes = dt_boxes
    for mf_box in mfdetrec_res:
        flag, left_box, right_box = False, None, None
        for idx, text_box in enumerate(new_dt_boxes):
            ret, left_box, right_box = formula_in_text(mf_box['bbox'], text_box)
            if ret:
                new_dt_boxes.pop(idx)
                if left_box is not None:
                    new_dt_boxes.append(left_box)
                if right_box is not None:
                    new_dt_boxes.append(right_box)
                break
            
    return new_dt_boxes

class ModifiedPaddleOCR(PaddleOCR):
    def ocr(self, img, det=True, rec=True, cls=True, bin=False, inv=False, mfd_res=None, alpha_color=(255, 255, 255)):
        """
        OCR with PaddleOCR
        args：
            img: img for OCR, support ndarray, img_path and list or ndarray
            det: use text detection or not. If False, only rec will be exec. Default is True
            rec: use text recognition or not. If False, only det will be exec. Default is True
            cls: use angle classifier or not. Default is True. If True, the text with rotation of 180 degrees can be recognized. If no text is rotated by 180 degrees, use cls=False to get better performance. Text with rotation of 90 or 270 degrees can be recognized even if cls=False.
            bin: binarize image to black and white. Default is False.
            inv: invert image colors. Default is False.
            alpha_color: set RGB color Tuple for transparent parts replacement. Default is pure white.
        """
        assert isinstance(img, (np.ndarray, list, str, bytes))
        if isinstance(img, list) and det == True:
            logger.error('When input a list of images, det must be false')
            exit(0)
        if cls == True and self.use_angle_cls == False:
            logger.warning(
                'Since the angle classifier is not initialized, it will not be used during the forward process'
            )

        img = check_img(img)
        # for infer pdf file
        if isinstance(img, list):
            if self.page_num > len(img) or self.page_num == 0:
                self.page_num = len(img)
            imgs = img[:self.page_num]
        else:
            imgs = [img]

        def preprocess_image(_image):
            _image = alpha_to_color(_image, alpha_color)
            if inv:
                _image = cv2.bitwise_not(_image)
            if bin:
                _image = binarize_img(_image)
            return _image

        if det and rec:
            ocr_res = []
            for idx, img in enumerate(imgs):
                img = preprocess_image(img)
                dt_boxes, rec_res, _ = self.__call__(img, cls, mfd_res=mfd_res)
                if not dt_boxes and not rec_res:
                    ocr_res.append(None)
                    continue
                tmp_res = [[box.tolist(), res]
                           for box, res in zip(dt_boxes, rec_res)]
                ocr_res.append(tmp_res)
            return ocr_res
        elif det and not rec:
            ocr_res = []
            for idx, img in enumerate(imgs):
                img = preprocess_image(img)
                dt_boxes, elapse = self.text_detector(img)
                if not dt_boxes:
                    ocr_res.append(None)
                    continue
                tmp_res = [box.tolist() for box in dt_boxes]
                ocr_res.append(tmp_res)
            return ocr_res
        else:
            ocr_res = []
            cls_res = []
            for idx, img in enumerate(imgs):
                if not isinstance(img, list):
                    img = preprocess_image(img)
                    img = [img]
                if self.use_angle_cls and cls:
                    img, cls_res_tmp, elapse = self.text_classifier(img)
                    if not rec:
                        cls_res.append(cls_res_tmp)
                rec_res, elapse = self.text_recognizer(img)
                ocr_res.append(rec_res)
            if not rec:
                return cls_res
            return ocr_res
        
    def __call__(self, img, cls=True, mfd_res=None):
        time_dict = {'det': 0, 'rec': 0, 'cls': 0, 'all': 0}

        if img is None:
            logger.debug("no valid image provided")
            return None, None, time_dict

        start = time.time()
        ori_im = img.copy()
        dt_boxes, elapse = self.text_detector(img)
        time_dict['det'] = elapse

        if dt_boxes is None:
            logger.debug("no dt_boxes found, elapsed : {}".format(elapse))
            end = time.time()
            time_dict['all'] = end - start
            return None, None, time_dict
        else:
            logger.debug("dt_boxes num : {}, elapsed : {}".format(
                len(dt_boxes), elapse))
        img_crop_list = []

        dt_boxes = sorted_boxes(dt_boxes)
        if mfd_res:
            bef = time.time()
            dt_boxes = update_det_boxes(dt_boxes, mfd_res)
            aft = time.time()
            logger.debug("split text box by formula, new dt_boxes num : {}, elapsed : {}".format(
                len(dt_boxes), aft-bef))

        for bno in range(len(dt_boxes)):
            tmp_box = copy.deepcopy(dt_boxes[bno])
            if self.args.det_box_type == "quad":
                img_crop = get_rotate_crop_image(ori_im, tmp_box)
            else:
                img_crop = get_minarea_rect_crop(ori_im, tmp_box)
            img_crop_list.append(img_crop)
        if self.use_angle_cls and cls:
            img_crop_list, angle_list, elapse = self.text_classifier(
                img_crop_list)
            time_dict['cls'] = elapse
            logger.debug("cls num  : {}, elapsed : {}".format(
                len(img_crop_list), elapse))

        rec_res, elapse = self.text_recognizer(img_crop_list)
        time_dict['rec'] = elapse
        logger.debug("rec_res num  : {}, elapsed : {}".format(
            len(rec_res), elapse))
        if self.args.save_crop_res:
            self.draw_crop_rec_res(self.args.crop_res_save_dir, img_crop_list,
                                   rec_res)
        filter_boxes, filter_rec_res = [], []
        for box, rec_result in zip(dt_boxes, rec_res):
            text, score = rec_result
            if score >= self.drop_score:
                filter_boxes.append(box)
                filter_rec_res.append(rec_result)
        end = time.time()
        time_dict['all'] = end - start
        return filter_boxes, filter_rec_res, time_dict


from flask import request
from flask_restful import Resource
from .serialization import MagicHtmlSchema
from marshmallow import ValidationError
from magic_doc.restful_api.common.custom_response import generate_response
from magic_html import GeneralExtractor
from loguru import logger

extractor = GeneralExtractor()


class MagicHtmlView(Resource):
    @logger.catch
    def post(self):
        """
        网页提取
        :return:
        """
        magic_html_schema = MagicHtmlSchema()
        try:
            params = magic_html_schema.load(request.get_json())
        except ValidationError as err:
            return generate_response(code=400, msg=err.messages)
        url = params.get("pageUrl", "")
        html_type = params.get("html_type")
        html = params.get("html")
        data = extractor.extract(html, base_url=url, html_type=html_type)
        return generate_response(data=data)



from pathlib import Path

from magic_pdf.dict2md.ocr_mkcontent import union_make
from magic_pdf.libs.MakeContentConfig import MakeMode, DropMode
from magic_pdf.libs.json_compressor import JsonCompressor
from magic_pdf.model.doc_analyze_by_pp_structurev2 import doc_analyze
from magic_pdf.rw.AbsReaderWriter import AbsReaderWriter

from loguru import logger

from magic_doc.conv.base import BaseConv
from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_doc.progress.pupdator import ConvProgressUpdator
from magic_doc.utils.null_writer import NullWriter

from magic_pdf.pipe.UNIPipe import UNIPipe

NULL_IMG_DIR = "/tmp"

class Pdf(BaseConv):
    def to_md(self, bits: bytes | str, pupdator: ConvProgressUpdator) -> str:

        model_list = doc_analyze(bits, ocr=False)
        pupdator.update(50)
        jso_useful_key = {
            "_pdf_type": "",
            "model_list": model_list,
        }
        image_writer = NullWriter()
        pipe = UNIPipe(bits, jso_useful_key, image_writer, is_debug=True)  # type: ignore
        pipe.pipe_classify() # 默认ocrpipe的时候不需要再做分类，可以节省时间
        pipe.pipe_parse()
        pupdator.update(100)

        pdf_mid_data = JsonCompressor.decompress_json(pipe.get_compress_pdf_mid_data())
        pdf_info_list = pdf_mid_data["pdf_info"]
        md_content = union_make(pdf_info_list, MakeMode.NLP_MD, DropMode.NONE, NULL_IMG_DIR)
        return md_content  # type: ignore

    def to_mid_result(self, image_writer: AbsReaderWriter, bits: bytes | str, pupdator: ConvProgressUpdator) \
            -> list[dict] | dict:

        pupdator.update(0)

        model_list = doc_analyze(bits)  # type: ignore
        pupdator.update(50)
        jso_useful_key = {
            "_pdf_type": "",
            "model_list": model_list,
        }

        pipe = UNIPipe(bits, jso_useful_key, image_writer, is_debug=True)  # type: ignore
        pipe.pipe_classify()
        pipe.pipe_parse()
        pupdator.update(100)

        pdf_mid_data = JsonCompressor.decompress_json(pipe.get_compress_pdf_mid_data())
        pdf_info_list = pdf_mid_data["pdf_info"]
        return pdf_info_list


if __name__ == "__main__":
    pupdator = FileBaseProgressUpdator("/tmp/p.txt")
    pdf = Pdf()
    logger.info(
        pdf.to_md(Path(r"D:\project\20231108code-clean\linshixuqiu\pdf_dev\新模型\j.sna.2004.11.030.pdf").read_bytes(), pupdator))



from setuptools import find_packages, setup
from pathlib import Path
from magic_doc.libs.version import __version__


def parse_requirements(filename):
    with open(filename) as f:
        lines = f.read().splitlines()

    requires = []

    for line in lines:
        if "http" in line:
            pkg_name_without_url = line.split("@")[0].strip()
            if pkg_name_without_url == "magic_pdf":
                pkg_name_without_url = "magic_pdf>=0.5.0"
            requires.append(pkg_name_without_url)
        else:
            requires.append(line)

    return requires


if __name__ == "__main__":
    with Path(Path(__file__).parent,
              'README.md').open(encoding='utf-8') as file:
        long_description = file.read()
    setup(
        name="fairy_doc",  # 项目名
        version=__version__,  # 自动从tag中获取版本号
        packages=find_packages() + ["magic_doc.bin", "magic_doc.resources"],  # 包含所有的包
        package_data={
            "magic_doc.bin": ["**"],  # 包含magic_doc.bin目录下的所有文件
            "magic_doc.resources": ["**"],  # 包含magic_doc.resources目录下的所有文件
            "magic_doc.contrib.office.formula": [
                "**"
            ],  # 包含magic_doc.contrib.office.formula目录下的所有文件
        },
        license='Apache 2.0',
        description='A lightweight toolbox to manipulate documents',
        long_description=long_description,
        long_description_content_type='text/markdown',
        install_requires=parse_requirements("requirements.txt"),  # 项目依赖的第三方库
        url="https://github.com/InternLM/magic-doc",
        python_requires=">=3.10",  # 项目依赖的 Python 版本
        # entry_points={"console_scripts": ["my_command=my_project.main:run"]}, # 项目提供的可执行命令
        include_package_data=True,
        zip_safe=False,  # 是否使用 zip 文件格式打包，一般设为 False
    )



import os
import tempfile
from pathlib import Path

from loguru import logger

from magic_doc.contrib.model import Page
from magic_doc.contrib.office.doc import DocExtractor
from magic_doc.conv.base import BaseConv
from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_doc.progress.pupdator import ConvProgressUpdator


class Doc(BaseConv):

    def __init__(self):
        super().__init__()

    def to_md(self, bits: bytes, pupdator:ConvProgressUpdator) -> str:
        page_list = self.doc_to_pagelist(bits, pupdator)
        md_content_list = []
        for page in page_list:
            page_content_list = page['content_list']
            total = len(page_content_list)
            for index, content in enumerate(page_content_list):
                progress = 50 + int(index / total * 50)
                pupdator.update(progress)
                if content['type'] == 'image':
                    pass
                elif content['type'] == "text":
                    data = content['data']
                    md_content_list.append(data)
        return "\n".join(md_content_list)

    def doc_to_pagelist(self, bits,  pupdator:ConvProgressUpdator) -> list[Page]:
        with tempfile.TemporaryDirectory() as temp_path:
            temp_dir = Path(temp_path)
            media_dir = temp_dir / "media"
            media_dir.mkdir()
            file_path = temp_dir / "tmp.doc"
            file_path.write_bytes(bits)
            doc_extractor = DocExtractor()
            cwd_path = os.path.dirname(os.path.abspath(__file__)) / Path("../bin/linux")
            bin_path = cwd_path / "antiword"
            os.chmod(bin_path, 0o755)
            page_list = doc_extractor.extract(file_path, "tmp", temp_dir, media_dir, True, cwd_path=cwd_path)
            pupdator.update(50)
        return page_list


if __name__ == '__main__':
    pupdator = FileBaseProgressUpdator("/tmp/p.txt")
    doc = Doc()
    logger.info(doc.to_md(Path("/home/myhloli/文本+表+图1.doc").read_bytes(), pupdator))



import torchvision.transforms.functional as F
import warnings
import math
import random
import numpy as np
from PIL import Image
import torch

from detectron2.data.detection_utils import read_image
from detectron2.data.transforms import ResizeTransform, TransformList

def normalize_bbox(bbox, size):
    return [
        int(1000 * bbox[0] / size[0]),
        int(1000 * bbox[1] / size[1]),
        int(1000 * bbox[2] / size[0]),
        int(1000 * bbox[3] / size[1]),
    ]


def load_image(image_path):
    image = read_image(image_path, format="BGR")
    h = image.shape[0]
    w = image.shape[1]
    img_trans = TransformList([ResizeTransform(h=h, w=w, new_h=224, new_w=224)])
    image = torch.tensor(img_trans.apply_image(image).copy()).permute(2, 0, 1)  # copy to make it writeable
    return image, (w, h)


def crop(image, i, j, h, w, boxes=None):
    cropped_image = F.crop(image, i, j, h, w)

    if boxes is not None:
        # Currently we cannot use this case since when some boxes is out of the cropped image,
        # it may be better to drop out these boxes along with their text input (instead of min or clamp)
        # which haven't been implemented here
        max_size = torch.as_tensor([w, h], dtype=torch.float32)
        cropped_boxes = torch.as_tensor(boxes) - torch.as_tensor([j, i, j, i])
        cropped_boxes = torch.min(cropped_boxes.reshape(-1, 2, 2), max_size)
        cropped_boxes = cropped_boxes.clamp(min=0)
        boxes = cropped_boxes.reshape(-1, 4)

    return cropped_image, boxes


def resize(image, size, interpolation, boxes=None):
    # It seems that we do not need to resize boxes here, since the boxes will be resized to 1000x1000 finally,
    # which is compatible with a square image size of 224x224
    rescaled_image = F.resize(image, size, interpolation)

    if boxes is None:
        return rescaled_image, None

    ratios = tuple(float(s) / float(s_orig) for s, s_orig in zip(rescaled_image.size, image.size))
    ratio_width, ratio_height = ratios

    # boxes = boxes.copy()
    scaled_boxes = boxes * torch.as_tensor([ratio_width, ratio_height, ratio_width, ratio_height])

    return rescaled_image, scaled_boxes


def clamp(num, min_value, max_value):
    return max(min(num, max_value), min_value)


def get_bb(bb, page_size):
    bbs = [float(j) for j in bb]
    xs, ys = [], []
    for i, b in enumerate(bbs):
        if i % 2 == 0:
            xs.append(b)
        else:
            ys.append(b)
    (width, height) = page_size
    return_bb = [
        clamp(min(xs), 0, width - 1),
        clamp(min(ys), 0, height - 1),
        clamp(max(xs), 0, width - 1),
        clamp(max(ys), 0, height - 1),
    ]
    return_bb = [
            int(1000 * return_bb[0] / width),
            int(1000 * return_bb[1] / height),
            int(1000 * return_bb[2] / width),
            int(1000 * return_bb[3] / height),
        ]
    return return_bb


class ToNumpy:

    def __call__(self, pil_img):
        np_img = np.array(pil_img, dtype=np.uint8)
        if np_img.ndim < 3:
            np_img = np.expand_dims(np_img, axis=-1)
        np_img = np.rollaxis(np_img, 2)  # HWC to CHW
        return np_img


class ToTensor:

    def __init__(self, dtype=torch.float32):
        self.dtype = dtype

    def __call__(self, pil_img):
        np_img = np.array(pil_img, dtype=np.uint8)
        if np_img.ndim < 3:
            np_img = np.expand_dims(np_img, axis=-1)
        np_img = np.rollaxis(np_img, 2)  # HWC to CHW
        return torch.from_numpy(np_img).to(dtype=self.dtype)


_pil_interpolation_to_str = {
    F.InterpolationMode.NEAREST: 'F.InterpolationMode.NEAREST',
    F.InterpolationMode.BILINEAR: 'F.InterpolationMode.BILINEAR',
    F.InterpolationMode.BICUBIC: 'F.InterpolationMode.BICUBIC',
    F.InterpolationMode.LANCZOS: 'F.InterpolationMode.LANCZOS',
    F.InterpolationMode.HAMMING: 'F.InterpolationMode.HAMMING',
    F.InterpolationMode.BOX: 'F.InterpolationMode.BOX',
}


def _pil_interp(method):
    if method == 'bicubic':
        return F.InterpolationMode.BICUBIC
    elif method == 'lanczos':
        return F.InterpolationMode.LANCZOS
    elif method == 'hamming':
        return F.InterpolationMode.HAMMING
    else:
        # default bilinear, do we want to allow nearest?
        return F.InterpolationMode.BILINEAR


class Compose:
    """Composes several transforms together. This transform does not support torchscript.
    Please, see the note below.

    Args:
        transforms (list of ``Transform`` objects): list of transforms to compose.

    Example:
        >>> transforms.Compose([
        >>>     transforms.CenterCrop(10),
        >>>     transforms.PILToTensor(),
        >>>     transforms.ConvertImageDtype(torch.float),
        >>> ])

    .. note::
        In order to script the transformations, please use ``torch.nn.Sequential`` as below.

        >>> transforms = torch.nn.Sequential(
        >>>     transforms.CenterCrop(10),
        >>>     transforms.Normalize((0.485, 0.456, 0.406), (0.229, 0.224, 0.225)),
        >>> )
        >>> scripted_transforms = torch.jit.script(transforms)

        Make sure to use only scriptable transformations, i.e. that work with ``torch.Tensor``, does not require
        `lambda` functions or ``PIL.Image``.

    """

    def __init__(self, transforms):
        self.transforms = transforms

    def __call__(self, img, augmentation=False, box=None):
        for t in self.transforms:
            img = t(img, augmentation, box)
        return img


class RandomResizedCropAndInterpolationWithTwoPic:
    """Crop the given PIL Image to random size and aspect ratio with random interpolation.
    A crop of random size (default: of 0.08 to 1.0) of the original size and a random
    aspect ratio (default: of 3/4 to 4/3) of the original aspect ratio is made. This crop
    is finally resized to given size.
    This is popularly used to train the Inception networks.
    Args:
        size: expected output size of each edge
        scale: range of size of the origin size cropped
        ratio: range of aspect ratio of the origin aspect ratio cropped
        interpolation: Default: PIL.Image.BILINEAR
    """

    def __init__(self, size, second_size=None, scale=(0.08, 1.0), ratio=(3. / 4., 4. / 3.),
                 interpolation='bilinear', second_interpolation='lanczos'):
        if isinstance(size, tuple):
            self.size = size
        else:
            self.size = (size, size)
        if second_size is not None:
            if isinstance(second_size, tuple):
                self.second_size = second_size
            else:
                self.second_size = (second_size, second_size)
        else:
            self.second_size = None
        if (scale[0] > scale[1]) or (ratio[0] > ratio[1]):
            warnings.warn("range should be of kind (min, max)")

        self.interpolation = _pil_interp(interpolation)
        self.second_interpolation = _pil_interp(second_interpolation)
        self.scale = scale
        self.ratio = ratio

    @staticmethod
    def get_params(img, scale, ratio):
        """Get parameters for ``crop`` for a random sized crop.
        Args:
            img (PIL Image): Image to be cropped.
            scale (tuple): range of size of the origin size cropped
            ratio (tuple): range of aspect ratio of the origin aspect ratio cropped
        Returns:
            tuple: params (i, j, h, w) to be passed to ``crop`` for a random
                sized crop.
        """
        area = img.size[0] * img.size[1]

        for attempt in range(10):
            target_area = random.uniform(*scale) * area
            log_ratio = (math.log(ratio[0]), math.log(ratio[1]))
            aspect_ratio = math.exp(random.uniform(*log_ratio))

            w = int(round(math.sqrt(target_area * aspect_ratio)))
            h = int(round(math.sqrt(target_area / aspect_ratio)))

            if w <= img.size[0] and h <= img.size[1]:
                i = random.randint(0, img.size[1] - h)
                j = random.randint(0, img.size[0] - w)
                return i, j, h, w

        # Fallback to central crop
        in_ratio = img.size[0] / img.size[1]
        if in_ratio < min(ratio):
            w = img.size[0]
            h = int(round(w / min(ratio)))
        elif in_ratio > max(ratio):
            h = img.size[1]
            w = int(round(h * max(ratio)))
        else:  # whole image
            w = img.size[0]
            h = img.size[1]
        i = (img.size[1] - h) // 2
        j = (img.size[0] - w) // 2
        return i, j, h, w

    def __call__(self, img, augmentation=False, box=None):
        """
        Args:
            img (PIL Image): Image to be cropped and resized.
        Returns:
            PIL Image: Randomly cropped and resized image.
        """
        if augmentation:
            i, j, h, w = self.get_params(img, self.scale, self.ratio)
            img = F.crop(img, i, j, h, w)
            # img, box = crop(img, i, j, h, w, box)
        img = F.resize(img, self.size, self.interpolation)
        second_img = F.resize(img, self.second_size, self.second_interpolation) \
            if self.second_size is not None else None
        return img, second_img

    def __repr__(self):
        if isinstance(self.interpolation, (tuple, list)):
            interpolate_str = ' '.join([_pil_interpolation_to_str[x] for x in self.interpolation])
        else:
            interpolate_str = _pil_interpolation_to_str[self.interpolation]
        format_string = self.__class__.__name__ + '(size={0}'.format(self.size)
        format_string += ', scale={0}'.format(tuple(round(s, 4) for s in self.scale))
        format_string += ', ratio={0}'.format(tuple(round(r, 4) for r in self.ratio))
        format_string += ', interpolation={0}'.format(interpolate_str)
        if self.second_size is not None:
            format_string += ', second_size={0}'.format(self.second_size)
            format_string += ', second_interpolation={0}'.format(_pil_interpolation_to_str[self.second_interpolation])
        format_string += ')'
        return format_string


def pil_loader(path: str) -> Image.Image:
    # open path as file to avoid ResourceWarning (https://github.com/python-pillow/Pillow/issues/835)
    with open(path, 'rb') as f:
        img = Image.open(f)
        return img.convert('RGB')



import os
import string

from lxml import etree as ET

from ..mml import mml2tex


_namespaces = {
    "wpc": "http://schemas.microsoft.com/office/word/2010/wordprocessingCanvas",
    "cx": "http://schemas.microsoft.com/office/drawing/2014/chartex",
    "cx1": "http://schemas.microsoft.com/office/drawing/2015/9/8/chartex",
    "mc": "http://schemas.openxmlformats.org/markup-compatibility/2006",
    "o": "urn:schemas-microsoft-com:office:office",
    "r": "http://schemas.openxmlformats.org/officeDocument/2006/relationships",
    "m": "http://schemas.openxmlformats.org/officeDocument/2006/math",
    "v": "urn:schemas-microsoft-com:vml",
    "wp14": "http://schemas.microsoft.com/office/word/2010/wordprocessingDrawing",
    "wp": "http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing",
    "w10": "urn:schemas-microsoft-com:office:word",
    "w": "http://schemas.openxmlformats.org/wordprocessingml/2006/main",
    "w14": "http://schemas.microsoft.com/office/word/2010/wordml",
    "w15": "http://schemas.microsoft.com/office/word/2012/wordml",
    "w16se": "http://schemas.microsoft.com/office/word/2015/wordml/symex",
    "wpg": "http://schemas.microsoft.com/office/word/2010/wordprocessingGroup",
    "wpi": "http://schemas.microsoft.com/office/word/2010/wordprocessingInk",
    "wne": "http://schemas.microsoft.com/office/word/2006/wordml",
    "wps": "http://schemas.microsoft.com/office/word/2010/wordprocessingShape",
}
_xmlns_str = " ".join(
    'xmlns:{}="{}"'.format(key, value) for key, value in _namespaces.items()
)

_template = string.Template(
    """<?xml version="1.0" standalone="yes"?>
<w:document mc:Ignorable="w14 w15 w16se wp14" {}>
    $omml_xml
</w:document>""".format(
        _xmlns_str
    )
)


transform = None

_xslt_filename = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "OMML2MML.XSL"
)


def omml2mml(omml_xml):
    xml_content = _template.safe_substitute(omml_xml=omml_xml)
    tree = ET.fromstring(xml_content)
    global transform
    if not transform:
        transform = ET.XSLT(ET.parse(_xslt_filename))
    return str(transform(tree))


def omml2tex(omml_xml):
    mml_xml = omml2mml(omml_xml)
    return mml2tex(mml_xml)





from magic_doc.model.sub_modules.layoutlmv3.model_init import Layoutlmv3_Predictor

class SeqLayout:
    def __init__(self, config):
        self.model = Layoutlmv3_Predictor(config)

    def __call__(self, params):
        """
        params: list[(idx, image)]
        """
        if len(params) == 0:
            return []

        results = []
        for idx, image in params:
            layout_res = self.model(image)
            results.append((idx, layout_res))
        return results




"""
Mostly copy-paste from DINO and timm library:
https://github.com/facebookresearch/dino
https://github.com/rwightman/pytorch-image-models/blob/master/timm/models/vision_transformer.py
"""
import warnings

import math
import torch
import torch.nn as nn
import torch.utils.checkpoint as checkpoint
from timm.models.layers import trunc_normal_, drop_path, to_2tuple
from functools import partial

def _cfg(url='', **kwargs):
    return {
        'url': url,
        'num_classes': 1000, 'input_size': (3, 224, 224), 'pool_size': None,
        'crop_pct': .9, 'interpolation': 'bicubic',
        'mean': (0.5, 0.5, 0.5), 'std': (0.5, 0.5, 0.5),
        **kwargs
    }

class DropPath(nn.Module):
    """Drop paths (Stochastic Depth) per sample  (when applied in main path of residual blocks).
    """

    def __init__(self, drop_prob=None):
        super(DropPath, self).__init__()
        self.drop_prob = drop_prob

    def forward(self, x):
        return drop_path(x, self.drop_prob, self.training)

    def extra_repr(self) -> str:
        return 'p={}'.format(self.drop_prob)


class Mlp(nn.Module):
    def __init__(self, in_features, hidden_features=None, out_features=None, act_layer=nn.GELU, drop=0.):
        super().__init__()
        out_features = out_features or in_features
        hidden_features = hidden_features or in_features
        self.fc1 = nn.Linear(in_features, hidden_features)
        self.act = act_layer()
        self.fc2 = nn.Linear(hidden_features, out_features)
        self.drop = nn.Dropout(drop)

    def forward(self, x):
        x = self.fc1(x)
        x = self.act(x)
        x = self.drop(x)
        x = self.fc2(x)
        x = self.drop(x)
        return x


class Attention(nn.Module):
    def __init__(self, dim, num_heads=8, qkv_bias=False, qk_scale=None, attn_drop=0., proj_drop=0.):
        super().__init__()
        self.num_heads = num_heads
        head_dim = dim // num_heads
        # NOTE scale factor was wrong in my original version, can set manually to be compat with prev weights
        self.scale = qk_scale or head_dim ** -0.5

        self.qkv = nn.Linear(dim, dim * 3, bias=qkv_bias)
        self.attn_drop = nn.Dropout(attn_drop)
        self.proj = nn.Linear(dim, dim)
        self.proj_drop = nn.Dropout(proj_drop)

    def forward(self, x):
        B, N, C = x.shape
        q, k, v = self.qkv(x).reshape(B, N, 3, self.num_heads,
                                      C // self.num_heads).permute(2, 0, 3, 1, 4)

        attn = (q @ k.transpose(-2, -1)) * self.scale
        attn = attn.softmax(dim=-1)
        attn = self.attn_drop(attn)

        x = (attn @ v).transpose(1, 2).reshape(B, N, C)
        x = self.proj(x)
        x = self.proj_drop(x)
        return x


class Block(nn.Module):

    def __init__(self, dim, num_heads, mlp_ratio=4., qkv_bias=False, qk_scale=None, drop=0., attn_drop=0.,
                 drop_path=0., act_layer=nn.GELU, norm_layer=nn.LayerNorm):
        super().__init__()
        self.norm1 = norm_layer(dim)
        self.attn = Attention(
            dim, num_heads=num_heads, qkv_bias=qkv_bias, qk_scale=qk_scale, attn_drop=attn_drop, proj_drop=drop)
        # NOTE: drop path for stochastic depth, we shall see if this is better than dropout here
        self.drop_path = DropPath(
            drop_path) if drop_path > 0. else nn.Identity()
        self.norm2 = norm_layer(dim)
        mlp_hidden_dim = int(dim * mlp_ratio)
        self.mlp = Mlp(in_features=dim, hidden_features=mlp_hidden_dim,
                       act_layer=act_layer, drop=drop)

    def forward(self, x):
        x = x + self.drop_path(self.attn(self.norm1(x)))
        x = x + self.drop_path(self.mlp(self.norm2(x)))
        return x


class PatchEmbed(nn.Module):
    """ Image to Patch Embedding
    """

    def __init__(self, img_size=224, patch_size=16, in_chans=3, embed_dim=768):
        super().__init__()
        img_size = to_2tuple(img_size)
        patch_size = to_2tuple(patch_size)

        self.window_size = (img_size[0] // patch_size[0], img_size[1] // patch_size[1])

        self.num_patches_w, self.num_patches_h = self.window_size

        self.num_patches = self.window_size[0] * self.window_size[1]
        self.img_size = img_size
        self.patch_size = patch_size

        self.proj = nn.Conv2d(in_chans, embed_dim,
                              kernel_size=patch_size, stride=patch_size)

    def forward(self, x):
        x = self.proj(x)
        return x


class HybridEmbed(nn.Module):
    """ CNN Feature Map Embedding
    Extract feature map from CNN, flatten, project to embedding dim.
    """

    def __init__(self, backbone, img_size=224, feature_size=None, in_chans=3, embed_dim=768):
        super().__init__()
        assert isinstance(backbone, nn.Module)
        img_size = to_2tuple(img_size)
        self.img_size = img_size
        self.backbone = backbone
        if feature_size is None:
            with torch.no_grad():
                # FIXME this is hacky, but most reliable way of determining the exact dim of the output feature
                # map for all networks, the feature metadata has reliable channel and stride info, but using
                # stride to calc feature dim requires info about padding of each stage that isn't captured.
                training = backbone.training
                if training:
                    backbone.eval()
                o = self.backbone(torch.zeros(
                    1, in_chans, img_size[0], img_size[1]))[-1]
                feature_size = o.shape[-2:]
                feature_dim = o.shape[1]
                backbone.train(training)
        else:
            feature_size = to_2tuple(feature_size)
            feature_dim = self.backbone.feature_info.channels()[-1]
        self.num_patches = feature_size[0] * feature_size[1]
        self.proj = nn.Linear(feature_dim, embed_dim)

    def forward(self, x):
        x = self.backbone(x)[-1]
        x = x.flatten(2).transpose(1, 2)
        x = self.proj(x)
        return x


class ViT(nn.Module):
    """ Vision Transformer with support for patch or hybrid CNN input stage
    """

    def __init__(self,
                 model_name='vit_base_patch16_224',
                 img_size=384,
                 patch_size=16,
                 in_chans=3,
                 embed_dim=1024,
                 depth=24,
                 num_heads=16,
                 num_classes=19,
                 mlp_ratio=4.,
                 qkv_bias=True,
                 qk_scale=None,
                 drop_rate=0.1,
                 attn_drop_rate=0.,
                 drop_path_rate=0.,
                 hybrid_backbone=None,
                 norm_layer=partial(nn.LayerNorm, eps=1e-6),
                 norm_cfg=None,
                 pos_embed_interp=False,
                 random_init=False,
                 align_corners=False,
                 use_checkpoint=False,
                 num_extra_tokens=1,
                 out_features=None,
                 **kwargs,
                 ):

        super(ViT, self).__init__()
        self.model_name = model_name
        self.img_size = img_size
        self.patch_size = patch_size
        self.in_chans = in_chans
        self.embed_dim = embed_dim
        self.depth = depth
        self.num_heads = num_heads
        self.num_classes = num_classes
        self.mlp_ratio = mlp_ratio
        self.qkv_bias = qkv_bias
        self.qk_scale = qk_scale
        self.drop_rate = drop_rate
        self.attn_drop_rate = attn_drop_rate
        self.drop_path_rate = drop_path_rate
        self.hybrid_backbone = hybrid_backbone
        self.norm_layer = norm_layer
        self.norm_cfg = norm_cfg
        self.pos_embed_interp = pos_embed_interp
        self.random_init = random_init
        self.align_corners = align_corners
        self.use_checkpoint = use_checkpoint
        self.num_extra_tokens = num_extra_tokens
        self.out_features = out_features
        self.out_indices = [int(name[5:]) for name in out_features]

        # self.num_stages = self.depth
        # self.out_indices = tuple(range(self.num_stages))

        if self.hybrid_backbone is not None:
            self.patch_embed = HybridEmbed(
                self.hybrid_backbone, img_size=self.img_size, in_chans=self.in_chans, embed_dim=self.embed_dim)
        else:
            self.patch_embed = PatchEmbed(
                img_size=self.img_size, patch_size=self.patch_size, in_chans=self.in_chans, embed_dim=self.embed_dim)
        self.num_patches = self.patch_embed.num_patches

        self.cls_token = nn.Parameter(torch.zeros(1, 1, self.embed_dim))

        if self.num_extra_tokens == 2:
            self.dist_token = nn.Parameter(torch.zeros(1, 1, self.embed_dim))

        self.pos_embed = nn.Parameter(torch.zeros(
            1, self.num_patches + self.num_extra_tokens, self.embed_dim))
        self.pos_drop = nn.Dropout(p=self.drop_rate)

        # self.num_extra_tokens = self.pos_embed.shape[-2] - self.num_patches
        dpr = [x.item() for x in torch.linspace(0, self.drop_path_rate,
                                                self.depth)]  # stochastic depth decay rule
        self.blocks = nn.ModuleList([
            Block(
                dim=self.embed_dim, num_heads=self.num_heads, mlp_ratio=self.mlp_ratio, qkv_bias=self.qkv_bias,
                qk_scale=self.qk_scale,
                drop=self.drop_rate, attn_drop=self.attn_drop_rate, drop_path=dpr[i], norm_layer=self.norm_layer)
            for i in range(self.depth)])

        # NOTE as per official impl, we could have a pre-logits representation dense layer + tanh here
        # self.repr = nn.Linear(embed_dim, representation_size)
        # self.repr_act = nn.Tanh()

        if patch_size == 16:
            self.fpn1 = nn.Sequential(
                nn.ConvTranspose2d(embed_dim, embed_dim, kernel_size=2, stride=2),
                nn.SyncBatchNorm(embed_dim),
                nn.GELU(),
                nn.ConvTranspose2d(embed_dim, embed_dim, kernel_size=2, stride=2),
            )

            self.fpn2 = nn.Sequential(
                nn.ConvTranspose2d(embed_dim, embed_dim, kernel_size=2, stride=2),
            )

            self.fpn3 = nn.Identity()

            self.fpn4 = nn.MaxPool2d(kernel_size=2, stride=2)
        elif patch_size == 8:
            self.fpn1 = nn.Sequential(
                nn.ConvTranspose2d(embed_dim, embed_dim, kernel_size=2, stride=2),
            )

            self.fpn2 = nn.Identity()

            self.fpn3 = nn.Sequential(
                nn.MaxPool2d(kernel_size=2, stride=2),
            )

            self.fpn4 = nn.Sequential(
                nn.MaxPool2d(kernel_size=4, stride=4),
            )

        trunc_normal_(self.pos_embed, std=.02)
        trunc_normal_(self.cls_token, std=.02)
        if self.num_extra_tokens==2:
            trunc_normal_(self.dist_token, std=0.2)
        self.apply(self._init_weights)
        # self.fix_init_weight()

    def fix_init_weight(self):
        def rescale(param, layer_id):
            param.div_(math.sqrt(2.0 * layer_id))

        for layer_id, layer in enumerate(self.blocks):
            rescale(layer.attn.proj.weight.data, layer_id + 1)
            rescale(layer.mlp.fc2.weight.data, layer_id + 1)

    def _init_weights(self, m):
        if isinstance(m, nn.Linear):
            trunc_normal_(m.weight, std=.02)
            if isinstance(m, nn.Linear) and m.bias is not None:
                nn.init.constant_(m.bias, 0)
        elif isinstance(m, nn.LayerNorm):
            nn.init.constant_(m.bias, 0)
            nn.init.constant_(m.weight, 1.0)

    '''
    def init_weights(self):
        logger = get_root_logger()

        trunc_normal_(self.pos_embed, std=.02)
        trunc_normal_(self.cls_token, std=.02)
        self.apply(self._init_weights)

        if self.init_cfg is None:
            logger.warn(f'No pre-trained weights for '
                        f'{self.__class__.__name__}, '
                        f'training start from scratch')
        else:
            assert 'checkpoint' in self.init_cfg, f'Only support ' \
                                                  f'specify `Pretrained` in ' \
                                                  f'`init_cfg` in ' \
                                                  f'{self.__class__.__name__} '
            logger.info(f"Will load ckpt from {self.init_cfg['checkpoint']}")
            load_checkpoint(self, filename=self.init_cfg['checkpoint'], strict=False, logger=logger)
    '''

    def get_num_layers(self):
        return len(self.blocks)

    @torch.jit.ignore
    def no_weight_decay(self):
        return {'pos_embed', 'cls_token'}

    def _conv_filter(self, state_dict, patch_size=16):
        """ convert patch embedding weight from manual patchify + linear proj to conv"""
        out_dict = {}
        for k, v in state_dict.items():
            if 'patch_embed.proj.weight' in k:
                v = v.reshape((v.shape[0], 3, patch_size, patch_size))
            out_dict[k] = v
        return out_dict

    def to_2D(self, x):
        n, hw, c = x.shape
        h = w = int(math.sqrt(hw))
        x = x.transpose(1, 2).reshape(n, c, h, w)
        return x

    def to_1D(self, x):
        n, c, h, w = x.shape
        x = x.reshape(n, c, -1).transpose(1, 2)
        return x

    def interpolate_pos_encoding(self, x, w, h):
        npatch = x.shape[1] - self.num_extra_tokens
        N = self.pos_embed.shape[1] - self.num_extra_tokens
        if npatch == N and w == h:
            return self.pos_embed

        class_ORdist_pos_embed = self.pos_embed[:, 0:self.num_extra_tokens]

        patch_pos_embed = self.pos_embed[:, self.num_extra_tokens:]

        dim = x.shape[-1]
        w0 = w // self.patch_embed.patch_size[0]
        h0 = h // self.patch_embed.patch_size[1]
        # we add a small number to avoid floating point error in the interpolation
        # see discussion at https://github.com/facebookresearch/dino/issues/8
        w0, h0 = w0 + 0.1, h0 + 0.1
        patch_pos_embed = nn.functional.interpolate(
            patch_pos_embed.reshape(1, int(math.sqrt(N)), int(math.sqrt(N)), dim).permute(0, 3, 1, 2),
            scale_factor=(w0 / math.sqrt(N), h0 / math.sqrt(N)),
            mode='bicubic',
        )
        assert int(w0) == patch_pos_embed.shape[-2] and int(h0) == patch_pos_embed.shape[-1]
        patch_pos_embed = patch_pos_embed.permute(0, 2, 3, 1).view(1, -1, dim)

        return torch.cat((class_ORdist_pos_embed, patch_pos_embed), dim=1)

    def prepare_tokens(self, x, mask=None):
        B, nc, w, h = x.shape
        # patch linear embedding
        x = self.patch_embed(x)

        # mask image modeling
        if mask is not None:
            x = self.mask_model(x, mask)
        x = x.flatten(2).transpose(1, 2)

        # add the [CLS] token to the embed patch tokens
        all_tokens = [self.cls_token.expand(B, -1, -1)]

        if self.num_extra_tokens == 2:
            dist_tokens = self.dist_token.expand(B, -1, -1)
            all_tokens.append(dist_tokens)
        all_tokens.append(x)

        x = torch.cat(all_tokens, dim=1)

        # add positional encoding to each token
        x = x + self.interpolate_pos_encoding(x, w, h)

        return self.pos_drop(x)

    def forward_features(self, x):
        # print(f"==========shape of x is {x.shape}==========")
        B, _, H, W = x.shape
        Hp, Wp = H // self.patch_size, W // self.patch_size
        x = self.prepare_tokens(x)

        features = []
        for i, blk in enumerate(self.blocks):
            if self.use_checkpoint:
                x = checkpoint.checkpoint(blk, x)
            else:
                x = blk(x)
            if i in self.out_indices:
                xp = x[:, self.num_extra_tokens:, :].permute(0, 2, 1).reshape(B, -1, Hp, Wp)
                features.append(xp.contiguous())

        ops = [self.fpn1, self.fpn2, self.fpn3, self.fpn4]
        for i in range(len(features)):
            features[i] = ops[i](features[i])

        feat_out = {}

        for name, value in zip(self.out_features, features):
            feat_out[name] = value

        return feat_out

    def forward(self, x):
        x = self.forward_features(x)
        return x


def deit_base_patch16(pretrained=False, **kwargs):
    model = ViT(
        patch_size=16,
        drop_rate=0.,
        embed_dim=768,
        depth=12,
        num_heads=12,
        num_classes=1000,
        mlp_ratio=4.,
        qkv_bias=True,
        use_checkpoint=True,
        num_extra_tokens=2,
        **kwargs)
    model.default_cfg = _cfg()
    return model

def mae_base_patch16(pretrained=False, **kwargs):
    model = ViT(
        patch_size=16,
        drop_rate=0.,
        embed_dim=768,
        depth=12,
        num_heads=12,
        num_classes=1000,
        mlp_ratio=4.,
        qkv_bias=True,
        use_checkpoint=True,
        num_extra_tokens=1,
        **kwargs)
    model.default_cfg = _cfg()
    return model


import os
from subprocess import Popen
import tempfile
from pathlib import Path

from loguru import logger

from magic_doc.contrib.model import Page
from magic_doc.contrib.office.pptx_extract import PptxExtractor
from magic_doc.conv.base import BaseConv
from magic_doc.progress.filepupdator import FileBaseProgressUpdator
from magic_doc.progress.pupdator import ConvProgressUpdator


class Ppt(BaseConv):
    def __init__(self):
        super().__init__()

    def to_md(self, bits: bytes, pupdator: ConvProgressUpdator) -> str:
        page_list = self.ppt_to_pagelist(bits, pupdator)
        md_content_list = []
        total = len(page_list)
        for index, page in enumerate(page_list):
            progress = 80 + int(index / total * 20)
            # logger.info(f"progress: {progress}")
            page_content_list = page['content_list']
            for content in page_content_list:
                pupdator.update(progress)
                if content['type'] == 'image':
                    pass
                elif content['type'] == "text":
                    data = content['data']
                    md_content_list.append(data)
        return "\n".join(md_content_list)

    def ppt_to_pptx(self, ppt_path: str, dir_path: str) -> str:
        cmd = f'soffice --headless --convert-to pptx "{ppt_path}" --outdir "{dir_path}"'
        logger.info(cmd)
        process = Popen(cmd, shell=True)
        process.wait()
        fname = str(Path(ppt_path).stem)
        pptx_path = os.path.join(os.path.dirname(ppt_path), f'{fname}.pptx')
        if not os.path.exists(pptx_path):
            # logger.error(f"> !!! File conversion failed {ppt_path} ==> {pptx_path}")
            raise Exception(f"> !!! File conversion failed {ppt_path} ==> {pptx_path}")
        else:
            return pptx_path

    def ppt_to_pagelist(self, bits, pupdator: ConvProgressUpdator) -> list[Page]:
        with tempfile.TemporaryDirectory() as temp_path:
            temp_dir = Path(temp_path)
            media_dir = temp_dir / "media"
            media_dir.mkdir()
            file_path = temp_dir / "tmp.ppt"
            file_path.write_bytes(bits)
            pptx_file_path = self.ppt_to_pptx(str(file_path), str(temp_path))
            pupdator.update(50)
            pptx_extractor = PptxExtractor()
            pages = pptx_extractor.extract(Path(pptx_file_path), "tmp", temp_dir, media_dir, True)
            pupdator.update(80)
            return pages


if __name__ == '__main__':
    pupdator = FileBaseProgressUpdator("/tmp/p.txt")
    ppt = Ppt()
    logger.info(
        ppt.to_md(
            open(r"D:\project\20240514magic_doc\doc_ppt\doc\【英文-课件】MIT15_082JF10_lec10.3MB.ppt", "rb").read(), pupdator))
